#include <sysint.h>

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdarg.h>


#include <mtypes.h>
#include <rf.h>
#include <vec.h>
#include <mat.h>
#include <rand.h>

#include <matop.h>




            /**********************/
            /*                    */
            /*    VECTOR NORMS    */
            /*                    */
            /**********************/


double vecnorm1(vector v)
    /* Vrne normo 1 vektorja v (vsoto abs. vred. komponent); Ce je v enak NULL
    ali ima dimenzijo 0, vrne 0.
    $A Igor jan00; */
{
int i;
double ret=0;
if (v!=NULL)
{
  for (i=1;i<=v->d;++i)
    ret+=fabs(v->v[i]);
}
return ret;
}

double vecnorm2(vector v)
    /* Vrne normo 2 vektorja v (koren vsote kvadratov komp.); Ce je v enak NULL
    ali ima dimenzijo 0, vrne 0.
    $A Igor jan00; */
{
int i;
double ret=0;
if (v!=NULL)
{
  for (i=1;i<=v->d;++i)
    ret+=v->v[i]*v->v[i];
}
ret=sqrt(ret);
return ret;
}

double vecnorminf(vector v)
    /* Vrne normo neskoncno vektorja v (najvec. abs. vred. komp.); Ce je v enak
    NULL ali ima dimenzijo 0, vrne 0.
    $A Igor jan00; */
{
int i;
double ret=0,x;
if (v!=NULL)
{
  for (i=1;i<=v->d;++i)
    if ((x=fabs(v->v[i]))>ret)
      ret=x;
}
return ret;
}

double vecnormE(vector v)
    /* Vrne evklidsko normo vektorja v (koren vsote kvadratov komp.); Ce je v
    enak NULL ali ima dimenzijo 0, vrne 0.
    $A Igor jan00; */
{
return vecnorm2(v);
}


double vecnorm(vector v)
    /* Vrne evklidsko normo oz. normo 2 vektorja v (koren vsote kvadratov
    komp.); Ce je v enak NULL ali ima dimenzijo 0, vrne 0.
    $A Igor jan00; */
{
int i;
double ret=0;
if (v!=NULL)
{
  for (i=1;i<=v->d;++i)
    ret+=v->v[i]*v->v[i];
}
ret=sqrt(ret);
return ret;
}


void normvecplain(vector v,vector res)
    /* Vector res postane normiran vektor v (uporabi se norma 2 oz. evklidska
    norma). res mora biti ze alociran z enako dimenzijo (vecjo od 0) kot
    v. res je lahko isti vektor kot v.
    $A Igor jan00; */
{
double norm;
int i;
norm=vecnorm(v);
for (i=1;i<=v->d;++i)
  res->v[i]=v->v[i]/norm;
}



vector normvec0(vector v1,vector *res)
    /* Vrne normiran vektor v1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. res je lahko tudi naslov v1.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi vektor, ki ga funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor jan00; */
{
vector m=NULL;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je vektor v1 kompatibilen z operacijo: */
  if (v1==NULL)
    dispvector(res);
  else if (v1->d<1)
    dispvector(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d!=v1->d)
        dispvector(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getvector(v1->d);
    /* Izvedba operacije: */
    normvecplain(v1,*res);
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova vektor, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je v1 kompatibilen z operacijo: */
  if (v1==NULL)
    return NULL;
  else if (v1->d<1)
    return NULL;
  else
  {
    m=getvector(v1->d);
    /* Izvedba operacije: */
    normvecplain(v1,m);
    return m;
  }
}
}



            /************************/
            /*                      */
            /*     MATRIX NORMS     */
            /*                      */
            /************************/



double matnorm1(matrix m)
    /* Vrne normo 1 matrike m (maksimum po stolpcih vsot absolutnih vrednosti
    komponent v danem stolpcu); Ce je m enaka NULL ali ima kaksno dimenzijo 0,
    vrne funkcija 0.
    $A Igor jan00; */
{
int i,j;
double ret=0,s;
if (m!=NULL)
  if (m->d1!=0 && m->d2!=0)
  {
    for (j=1;j<=m->d2;++j)
    {
      /* Vsota abs. vred. komponent v stolpcu j: */
      s=0;
      for (i=1;i<=m->d1;++i)
        s+=fabs(m->m[i][j]);
      /* Ce je vsota vecja od dozdaj najvecje, nadomesti ret: */
      if (s>ret)
        ret=s;
    }
  }
return ret;
}



double matnorminf(matrix m)
    /* Vrne normo neskoncno matrike m (maksimum po vrsticah vsot absolutnih
    vrednosti komponent v dani vrstici). Ce je m enaka NULL ali ima kaksno
    dimenzijo 0, vrne funkcija 0.
    $A Igor jan00; */
{
int i,j;
double ret=0,s;
if (m!=NULL)
  if (m->d1!=0 && m->d2!=0)
  {
    for (i=1;i<=m->d1;++i)
    {
      /* Vsota abs. vred. komponent v stolpcu j: */
      s=0;
      for (j=1;j<=m->d2;++j)
        s+=fabs(m->m[i][j]);
      /* Ce je vsota vecja od dozdaj najvecje, nadomesti ret: */
      if (s>ret)
        ret=s;
    }
  }
return ret;
}


double matnormE(matrix m)
    /* Vrne evklidsko normo matrike m (koren iz vsote kvadratov komponent). Ce
    je m enaka NULL ali ima kaksno
    dimenzijo 0, vrne funkcija 0.
    $A Igor jan00; */
{
int i,j;
double ret=0;
if (m!=NULL)
  if (m->d1!=0 && m->d2!=0)
  {
    for (i=1;i<=m->d1;++i)
    {
      for (j=1;j<=m->d2;++j)
        ret+=m->m[i][j]*m->m[i][j];
    }
  }
ret=sqrt(ret);
return ret;
}


double matnorm(matrix m)
    /* Vrne evklidsko normo matrike m (koren iz vsote kvadratov komponent). Ce
    je m enaka NULL ali ima kaksno
    dimenzijo 0, vrne funkcija 0.
    $A Igor jan00; */
{
int i,j;
double ret=0;
if (m!=NULL)
  if (m->d1!=0 && m->d2!=0)
  {
    for (i=1;i<=m->d1;++i)
    {
      for (j=1;j<=m->d2;++j)
        ret+=m->m[i][j]*m->m[i][j];
    }
  }
ret=sqrt(ret);
return ret;
}



void normmatplain(matrix m,matrix res)
    /* Matrika res postane normirana matrika m (uporabi se evklidska norma).
    res mora biti ze alocirana z enakimi dimenzijami (vecjimi od 0) kot
    m. res je lahko ista matrika kot m.
    $A Igor jan00; */
{
double norm;
int i,j;
norm=matnorm(m);
for (i=1;i<=m->d1;++i)
  for (j=1;j<=m->d2;++j)
    res->m[i][j]=m->m[i][j]/norm;
}


matrix normmat0(matrix m1,matrix *res)
    /* Vrne normirano matriko m1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. res je lahko tudi naslov m1.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (m1->d1<1 || m1->d2<1)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=m1->d1 || (*res)->d2!=m1->d2)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    normmatplain(m1,*res);
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    normmatplain(m1,m);
    return m;
  }
}
}



            /*******************************************/
            /*                                         */
            /*  CONVERSION BETWEEN MATRIX AND VECTORS  */
            /*                                         */
            /*******************************************/



matrix vectomat0(vector v1,matrix *res)
    /* Vrne matricno kopijo (stolpec) vektorja v1. Ce je res razlicen od NULL,
    zapise rezultat v *res in vrne *res.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL;
int i;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je vektor v1 kompatibilen z operacijo: */
  if (v1==NULL)
    dispmatrix(res);
  else if (v1->d<1)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=v1->d || (*res)->d2!=1)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(v1->d,1);
    /* Izvedba operacije: */
    for (i=1;i<=v1->d;++i)
      (*res)->m[i][1]=v1->v[i];
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je v1 kompatibilen z operacijo: */
  if (v1==NULL)
    return NULL;
  else if (v1->d<1)
    return NULL;
  else
  {
    m=getmatrix(v1->d,1);
    /* Izvedba operacije: */
    for (i=1;i<=v1->d;++i)
      m->m[i][1]=v1->v[i];
    return m;
  }
}
}



matrix vectransptomat0(vector v1,matrix *res)
    /* Vrne matricno kopijo (vrstico) transponiranega vektorja v1. Ce je res
    razlicen od NULL, zapise rezultat v *res in vrne *res.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL;
int i;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je vektor v1 kompatibilen z operacijo: */
  if (v1==NULL)
    dispmatrix(res);
  else if (v1->d<1)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=1 || (*res)->d2!=v1->d)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(1,v1->d);
    /* Izvedba operacije: */
    for (i=1;i<=v1->d;++i)
      (*res)->m[1][i]=v1->v[i];
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je v1 kompatibilen z operacijo: */
  if (v1==NULL)
    return NULL;
  else if (v1->d<1)
    return NULL;
  else
  {
    m=getmatrix(1,v1->d);
    /* Izvedba operacije: */
    for (i=1;i<=v1->d;++i)
      m->m[1][i]=v1->v[i];
    return m;
  }
}
}



vector mattovec0(matrix m1,vector *res)
    /* Vrne vektorsko kopijo matrike m1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. Matrika m1 se v vektor prepise po vrsticah.
    $A Igor feb00; */
{
vector v=NULL;
int i,j,k;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispvector(res);
  else if (m1->d1<1 || m1->d2<1)
    dispvector(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d!=m1->d1*m1->d2)
        dispvector(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getvector(m1->d1*m1->d2);
    /* Izvedba operacije: */
    k=1;
    for (i=1;i<=m1->d1;++i)
      for (j=1;j<=m1->d2;++j)
      {
        (*res)->v[k]=m1->m[i][j];
        ++k;
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nov vektor, v katero se zapise rezultat operacije,
  vrne se njgov kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    v=getvector(m1->d1*m1->d2);
    /* Izvedba operacije: */
    k=1;
    for (i=1;i<=m1->d1;++i)
      for (j=1;j<=m1->d2;++j)
      {
        v->v[k]=m1->m[i][j];
        ++k;
      }
    return v;
  }
}
}


vector mattransptovec0(matrix m1,vector *res)
    /* Vrne vektorsko kopijo (transponirane) matrike m1. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. Matrika m1 se v vektor prepise
    po stolpcih.
    $A Igor feb00; */
{
vector v=NULL;
int i,j,k;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispvector(res);
  else if (m1->d1<1 || m1->d2<1)
    dispvector(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d!=m1->d1*m1->d2)
        dispvector(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getvector(m1->d1*m1->d2);
    /* Izvedba operacije: */
    k=1;
    for (j=1;j<=m1->d2;++j)
      for (i=1;i<=m1->d1;++i)
      {
        (*res)->v[k]=m1->m[i][j];
        ++k;
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nov vektor, v katero se zapise rezultat operacije,
  vrne se njgov kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    v=getvector(m1->d1*m1->d2);
    /* Izvedba operacije: */
    k=1;
    for (j=1;j<=m1->d2;++j)
      for (i=1;i<=m1->d1;++i)
      {
        v->v[k]=m1->m[i][j];
        ++k;
      }
    return v;
  }
}
}



matrix matparttomat0(matrix m1,int i1,int j1,int i2,int j2,matrix *res)
    /* Vrne matricno kopijo dela matrike m1 po vrsticah. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. i1, j1, i2 in j2 dolocajo del
    matrike, ki se prepise v rezultat (tj. od vkljucno elementa (i1,j1) do
    vkljucno elementa (i2,j2)).
    $A Igor feb00; */
{
matrix m=NULL;
int i,j;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=i2-i1+1 || (*res)->d2!=j2-j1+1)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(i2-i1+1,j2-j1+1);
    /* Izvedba operacije: */
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        (*res)->m[i-i1+1][j-j1+1]=m1->m[i][j];
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    return NULL;
  else
  {
    m=getmatrix(i2-i1+1,j2-j1+1);
    /* Izvedba operacije: */
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        m->m[i-i1+1][j-j1+1]=m1->m[i][j];
      }
    return m;
  }
}
}


matrix matparttransptomat0(matrix m1,int i1,int j1,int i2,int j2,matrix *res)
    /* Vrne matricno kopijo dela matrike m1 po stolpcih. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. i1, j1, i2 in j2 dolocajo del
    matrike, ki se prepise v rezultat (tj. od vkljucno elementa (i1,j1) do
    vkljucno elementa (i2,j2)). Del matrike se prepisuje po stolpcih v vrstice.
    $A Igor feb00; */
{
matrix m=NULL;
int i,j;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=j2-j1+1 || (*res)->d2!=i2-i1+1)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(j2-j1+1,i2-i1+1);
    /* Izvedba operacije: */
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        (*res)->m[j-j1+1][i-i1+1]=m1->m[i][j];
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    return NULL;
  else
  {
    m=getmatrix(j2-j1+1,i2-i1+1);
    /* Izvedba operacije: */
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        m->m[j-j1+1][i-i1+1]=m1->m[i][j];
      }
    return m;
  }
}
}




vector matparttovec0(matrix m1,int i1,int j1,int i2,int j2,vector *res)
    /* Vrne vektorsko kopijo dela matrike m1 po vrsticah. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. i1, j1, i2 in j2 dolocajo del
    matrike, ki se prepise v rezultat (tj. od vkljucno elementa (i1,j1) do
    vkljucno elementa (i2,j2)).
    $A Igor feb00; */
{
vector v=NULL;
int i,j,k;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispvector(res);
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    dispvector(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d!=(i2-i1+1)*(j2-j1+1))
        dispvector(res);
    /* Ce je *res==NULL, ga tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getvector((i2-i1+1)*(j2-j1+1));
    /* Izvedba operacije: */
    k=1;
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        (*res)->v[k]=m1->m[i][j];
        ++k;
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nov vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    return NULL;
  else
  {
    v=getvector((i2-i1+1)*(j2-j1+1));
    /* Izvedba operacije: */
    for (i=i1;i<=i2;++i)
      for (j=j1;j<=j2;++j)
      {
        v->v[k]=m1->m[i][j];
        ++k;
      }
    return v;
  }
}
}


vector matrowtovec0(matrix m1,int ln,vector *res)
    /* Returns a vector copy of the line ln of matrix m1. If res is not NULL
    then the copy is stored to *res and *res is returned.
    $A Igor feb00; */
{
if (m1!=NULL)
  return matparttovec0(m1,ln,1,ln,m1->d2,res);
else
{
  dispvector(res);
  return NULL;
}
}


vector matcoltovec0(matrix m1,int col,vector *res)
    /* Returns a vector copy of the column col of matrix m1. If res is not NULL
    then the copy is stored to *res and *res is returned.
    $A Igor feb00; */
{
if (m1!=NULL)
  return matparttovec0(m1,1,col,m1->d1,col,res);
else
{
  dispvector(res);
  return NULL;
}
}


vector matparttransptovec0(matrix m1,int i1,int j1,int i2,int j2,vector *res)
    /* Vrne vektorsko kopijo dela matrike m1 po stolpcih. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. i1, j1, i2 in j2 dolocajo del
    matrike, ki se prepise v rezultat (tj. od vkljucno elementa (i1,j1) do
    vkljucno elementa (i2,j2)).
    $A Igor feb00; */
{
vector v=NULL;
int i,j,k;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispvector(res);
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    dispvector(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d!=(i2-i1+1)*(j2-j1+1))
        dispvector(res);
    /* Ce je *res==NULL, ga tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getvector((i2-i1+1)*(j2-j1+1));
    /* Izvedba operacije: */
    k=1;
    for (j=j1;j<=j2;++j)
      for (i=i1;i<=i2;++i)
      {
        (*res)->v[k]=m1->m[i][j];
        ++k;
      }
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nov vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (i1<1 || j1<1 || i2<i1 || j2<j1 || m1->d1<i2 || m1->d2<j2)
    return NULL;
  else
  {
    v=getvector((i2-i1+1)*(j2-j1+1));
    /* Izvedba operacije: */
    for (j=i1;j<=i2;++j)
      for (i=i1;i<=i2;++i)
      {
        v->v[k]=m1->m[i][j];
        ++k;
      }
    return v;
  }
}
}




            /*********************************/
            /*                               */
            /*   INITIALIZATION OF VECTORS   */
            /*                               */
            /*********************************/


vector initvec0(vector *v,int d,double comp1,...)
    /* Creates and returns a vector with a given set of components which must
    be listed after v. All components of the vector must be listed (their
    number is specified by d) and must be of type double. This function is
    convenient for initialization of smaller or mid-size vectors with known
    components.
      If v!=NULL then *v is initialized. In any case the pointer of vector
    is returned. If v is NULL then a vector is created anew and returned.
      WARNING:
      The caller must make sure that actual arguments that follow comp1 are
    indeed of type double (e.g. by cast). If components are integers they
    should be followed by ., for example 2., 3., 15.! The most common error is
    listing integers as integers, e.g. 2, 3, 15. In this case conversion is
    not performed implicitly as usually because at the caller's side the type
    of arguments following comp11 is not determined!
    $A Igor nov03; */
{
vector ret=NULL;
va_list ap;
int i;
if (d>0)
{
  if (v!=NULL)
  {
    if (*v==NULL)
      *v=getvector(d);
    else if ((*v)->d!=d)
    {
      dispvector(v);
      *v=getvector(d);
    }
    ret=*v;
  } else
    ret=getvector(d);
  va_start(ap,comp1);
  ret->v[1]=comp1;
  i=2;
  while(i<=d)
  {
    ret->v[i]=va_arg(ap,double);
    ++i;
  }
} else
  dispvector(v);
return ret;
}


vector randvec0(int d,int which,double par1,double par2,vector *v)
    /* Naredi in vrne vektor z nakljucnimi elementi. d je dimenzija vektorja,
    which doloca, po kaksni porazdelitvi so porazdeljeni elementi, par1 in par2
    sta parametra verjet. porazdelitve (npr. pri Gaussovi).
     Ce je d 0, postane d dimenzija vektorja *v, ce je ta razlicen od NULL.
    Vektor se vrne tudi v *v, ce je v razlicen od NULL.
     Zaenkrat so implementirane naslednje moznosti glede na vrednosti argumenta
    which:
     0,1: enakomer. porazdelitev med 0 in 1
     2:   enakomer. porazdelitev med par1 in par2
     3:   enakomer. porazdelitev med -par1 in par1
    $A Igor jul00 */
{
vector ret=NULL;
int i;
if (d==0 && v!=NULL) /* privzame se dimenzija *v */
  if (*v!=NULL)
    d=(*v)->d;
if (d>0)
{
  if (v!=NULL)
  {
    if (*v==NULL)
      *v=getvector(d);
    else if ((*v)->d!=d)
    {
      dispvector(v);
      *v=getvector(d);
    }
    ret=*v;
  } else
    ret=getvector(d);
  if (which==2)
  {
    /* Enakomerna porazdelitev med par1 in par2: */
    for (i=1;i<=d;++i)
      ret->v[i]=par1+random1()*(par2-par1);
  } else if (which==3)
  {
    /* Enakomerna porazdelitev med -par1 in par1: */
    for (i=1;i<=d;++i)
      ret->v[i]=-par1+random1()*2*par1;
  } else  /* which=1,0 */
  {
    /* Enakomerna porazdelitev med 0 in 1: */
    for (i=1;i<=d;++i)
      ret->v[i]=random1();
  }  
} else
  dispvector(v);
return ret;
}



vector constvec0(int d,double c,vector *v)
    /* Naredi in vrne vektor z elementi enakimi c. d je dimenzija vektorja.
    Ce je d 0, postane d dimenzija vektorja *v, ce je ta razlicen od NULL.
    Vektor se vrne tudi v *v, ce je v razlicen od NULL.
    $A Igor jul00 */
{
vector ret=NULL;
int i;
if (d==0 && v!=NULL) /* privzame se dimenzija *v */
  if (*v!=NULL)
    d=(*v)->d;
if (d>0)
{
  if (v!=NULL)
  {
    if (*v==NULL)
      *v=getvector(d);
    else if ((*v)->d!=d)
    {
      dispvector(v);
      *v=getvector(d);
    }
    ret=*v;
  } else
    ret=getvector(d);
  for (i=1;i<=d;++i)
    ret->v[i]=c;
} else
  dispvector(v);
return ret;
}


vector zerovec0(int d,vector *v)
    /* Naredi in vrne vektor z elementi enakimi 0. d je dimenzija vektorja.
    Ce je d 0, postane d dimenzija vektorja *v, ce je ta razlicen od NULL.
    Vektor se vrne tudi v *v, ce je v razlicen od NULL.
    $A Igor jul00 */
{
return constvec0(d,0,v);
}


vector unitvec0(int d,int which,vector *v)
    /* Naredi in vrne which-ti enotski vektor (elementi 0 razen na mestu which,
    kjer je element enak 1). d je dimenzija vektorja.
    Ce je d 0, postane d dimenzija vektorja *v, ce je ta razlicen od NULL.
    Vektor se vrne tudi v *v, ce je v razlicen od NULL.
    $A Igor jul00 */
{
vector ret=NULL;
int i;
if (d==0 && v!=NULL) /* privzame se dimenzija *v */
  if (*v!=NULL)
    d=(*v)->d;
if (d>0 && which>0 && which<=d)
{
  if (v!=NULL)
  {
    if (*v==NULL)
      *v=getvector(d);
    else if ((*v)->d!=d)
    {
      dispvector(v);
      *v=getvector(d);
    }
    ret=*v;
  } else
    ret=getvector(d);
  for (i=1;i<=d;++i)
    ret->v[i]=0;
  ret->v[which]=1;
} else
  dispvector(v);
return ret;
}





            /**********************************/
            /*                                */
            /*   INITIALIZATION OF MATRICES   */
            /*                                */
            /**********************************/



matrix initmat0(matrix *m,int d1,int d2,double comp11,...)
    /* Creates and returns a matrix with a given set of components which must
    be listed after m. All components of the matrix must be listed (their
    number is d1*d2) and must be of type double. This function is convenient
    for initialization of smaller or mid-size matrices with known components.
      If m!=NULL then *m is initialized. In any case the pointer of the matrix
    is returned. If m is NULL then a matrix is created anew and returned. If
    d1 or d2 is 0 then a NULL matrix is created.
      WARNING:
    The caller must make sure that actual arguments that follow comp11 are
    indeed of type double (e.g. by cast). If components are integers they
    should be followed by ., for example 2., 3., 15.! The most common error is
    listing integers as integers, e.g. 2, 3, 15. In this case conversion is
    not performed implicitly as usually because at the caller's side the type
    of arguments following comp11 is not determined!
    $A Igor nov03; */
{
matrix ret=NULL;
int i,j;
va_list ap;
if (d2==0)
  d2=d1;
if (d1>0 && d2>0)
{
  if (m!=NULL)
  {
    if (*m==NULL)
      *m=getmatrix(d1,d2);
    else if ((*m)->d1!=d1 || (*m)->d2!=d2)
    {
      dispmatrix(m);
      *m=getmatrix(d1,d2);
    }
    ret=*m;
  } else
    ret=getmatrix(d1,d2);
  va_start(ap,comp11);
  ret->m[1][1]=comp11;
  i=1; j=2;
  while(i<=d1)
  {
    while(j<=d2)
    {
      ret->m[i][j]=va_arg(ap,double);
      ++j;
    }
    ++i;
    j=1;
  }
} else
  dispmatrix(m);
return ret;
}




matrix randmat0(int d1,int d2,int which,double par1,double par2,
       matrix *m)
    /* Naredi in vrne matriko z nakljucnimi elementi. d1 in d2 sta dimenziji
    matrike, par1 in par2 sta parametra verjet. porazdelitve (npr. pri
    Gaussovi), which doloca, po kaksni porazdelitvi so porazdeljeni elementi.
    Ce je d2 0, postane d2 enak d1 (kvadratna matrika), ce pa sta d2 in d1
    enaka 0, postaneta dimenziji matrike *m, ce je ta razlicna od NULL.
    Zaenkrat so implementirane naslednje moznosti glede na vrednosti argumenta
    which:
     0,1: enakomer. porazdelitev med 0 in 1
     2:   enakomer. porazdelitev med par1 in par2
     3:   enakomer. porazdelitev med -par1 in par1
    $A Igor jan00 */
{
matrix ret=NULL;
int i,j;
if (d2==0)
  d2=d1;
if (d2==0 && m!=NULL) /* Torej d2 in d1 enaka 0, privzamejo se dimenzije *m */
  if (*m!=NULL)
  {
    d1=(*m)->d1;
    d2=(*m)->d2;
  }
if (d1>0 && d2>0)
{
  if (m!=NULL)
  {
    if (*m==NULL)
      *m=getmatrix(d1,d2);
    else if ((*m)->d1!=d1 || (*m)->d2!=d2)
    {
      dispmatrix(m);
      *m=getmatrix(d1,d2);
    }
    ret=*m;
  } else
    ret=getmatrix(d1,d2);
  if (which==2)
  {
    /* Enakomerna porazdelitev med par1 in par2: */
    for (i=1;i<=d1;++i)
      for (j=1;j<=d2;++j)
        ret->m[i][j]=par1+random1()*(par2-par1);
  } else if (which==3)
  {
    /* Enakomerna porazdelitev med -par1 in par1: */
    for (i=1;i<=d1;++i)
      for (j=1;j<=d2;++j)
        ret->m[i][j]=-par1+random1()*2*par1;
  } else  /* which=1 */
  {
    /* Enakomerna porazdelitev med 0 in 1: */
    for (i=1;i<=d1;++i)
      for (j=1;j<=d2;++j)
        ret->m[i][j]=random1();
  }  
} else
  dispmatrix(m);
return ret;
}




matrix constmat0(int d1,int d2,double c,matrix *m)
    /* Elemente matrike dimenzij d1*d2 postavi na d in vrne matriko. Ce je m
    razlicna od NULL, postane *m ta matrika, drugace se matrika tvori na novo.
    Ce je d2 0, postane d2 enak d1 (kvadratna matrika), ce pa sta d2 in d1
    enaka 0, postaneta dimenziji matrike *m, ce je ta razlicna od NULL.
    $A Igor jan00; */
{
matrix ret=NULL;
int i,j;
if (d2==0)
  d2=d1;
if (d2==0 && m!=NULL) /* Torej d2 in d1 enaka 0, privzamejo se dimenzije *m */
  if (*m!=NULL)
  {
    d1=(*m)->d1;
    d2=(*m)->d2;
  }
if (d1>0 && d2>0)
{
  if (m!=NULL)
  {
    if (*m==NULL)
      *m=getmatrix(d1,d2);
    else if ((*m)->d1!=d1 || (*m)->d2!=d2)
    {
      dispmatrix(m);
      *m=getmatrix(d1,d2);
    }
    ret=*m;
  } else
    ret=getmatrix(d1,d2);
  for (i=1;i<=d1;++i)
    for (j=1;j<=d2;++j)
      ret->m[i][j]=c;
} else
  dispmatrix(m);
return ret;
}


matrix zeromat0(int d1,int d2,matrix *m)
    /* Elemente matrike dimenzij d1*d2 postavi na 0 in vrne matriko. Ce je m
    razlicna od NULL, postane *m ta matrika, drugace se matrika tvori na novo.
    Ce je d2 0, postane d2 enak d1 (kvadratna matrika), ce pa sta d2 in d1
    enaka 0, postaneta dimenziji matrike *m, ce je ta razlicna od NULL.
    $A Igor jan00; */
{
return constmat0(d1,d2,0,m);
}


matrix identmat0(int d1,matrix *m)
    /* Naredi identicno kvadratno matriko dimenzij d1*d1 in vrne matriko. Ce
    je m razlicna od NULL, postane *m ta matrika, drugace se matrika tvori na
    novo.
    Ce je d1 enak 0, postane dimenzija matrike kar dimenzija *m, ce m kaze na
    kvadratno matriko.
    $A Igor jan00; */
{
matrix ret=NULL;
int i,j;
if (d1==0 && m!=NULL)
  if (*m!=NULL)
    if ((*m)->d1==(*m)->d2)
      d1=(*m)->d1;
if (d1>0)
{
  if (m!=NULL)
  {
    if (*m==NULL)
      *m=getmatrix(d1,d1);
    else if ((*m)->d1!=d1 || (*m)->d2!=d1)
    {
      dispmatrix(m);
      *m=getmatrix(d1,d1);
    }
    ret=*m;
  } else
    ret=getmatrix(d1,d1);
  for (i=1;i<=d1;++i)
    for (j=1;j<=d1;++j)
    {
      if (i==j)
        ret->m[i][j]=1;
      else
        ret->m[i][j]=0;
    }
} else
  dispmatrix(m);
return ret;
}


matrix diagmatconst0(int d1,double c,matrix *m)
    /* Naredi diagonalno kvadratno matriko dimenzij d1*d1 z diagonalnimi
    elementi enakimi c in vrne matriko. Ce je m razlicna od NULL, postane *m ta
    matrika, drugace se matrika tvori na novo.
    Ce je d1 enak 0, postane dimenzija matrike kar dimenzija *m, ce m kaze na
    kvadratno matriko.
    $A Igor jul00; */
{
matrix ret=NULL;
int i,j;
if (d1==0 && m!=NULL)
  if (*m!=NULL)
    if ((*m)->d1==(*m)->d2)
      d1=(*m)->d1;
if (d1>0)
{
  if (m!=NULL)
  {
    if (*m==NULL)
      *m=getmatrix(d1,d1);
    else if ((*m)->d1!=d1 || (*m)->d2!=d1)
    {
      dispmatrix(m);
      *m=getmatrix(d1,d1);
    }
    ret=*m;
  } else
    ret=getmatrix(d1,d1);
  for (i=1;i<=d1;++i)
    for (j=1;j<=d1;++j)
    {
      if (i==j)
        ret->m[i][j]=c;
      else
        ret->m[i][j]=0;
    }
} else
  dispmatrix(m);
return ret;
}







            /***********************************************/
            /*                                             */
            /*  SUMS AND PRODUCTS OF MATRICES AND VECTORS  */
            /*                                             */
            /***********************************************/




    /* SUMS AND PRODUCTS: VECTORS */


void vecsumplain(vector v1,vector v2,vector res)
    /* V vektor res zapise vsoto vektorjev v1 in v2. Funkcija ne kontrolira
    niti dimenzij niti tega, ce je prostor alociran. Vektor res je lahko isti
    kot v1 ali v2, prav tako sta v1 in v2 lahko ista vektorja.
    $A Igor jul00; */
{
int i;
for (i=1;i<=v1->d;++i)
  res->v[i]=v1->v[i]+v2->v[i];
}

void vecdifplain(vector v1,vector v2,vector res)
    /* V vektor res zapise razliko vektorjev v1 in v2. Funkcija ne kontrolira
    niti dimenzij niti tega, ce je prostor alociran. Vektor res je lahko isti
    kot v1 ali v2, prav tako sta v1 in v2 lahko ista vektorja.
    $A Igor jul00; */
{
int i;
for (i=1;i<=v1->d;++i)
  res->v[i]=v1->v[i]-v2->v[i];
}

vector vecsum0(vector v1,vector v2,vector *v3)
    /* Stores a sum of v1 and v2 to *v3 and returns *v3. If v3=NULL then the
    sum is created anew and returned. *v3 can point to v1 or v2, which can be
    the same vectors. If dimensions of *v3 do not match then v3 is reallocated
    first.
    $A jul00; */
{
return vectorsum0(v1,v2,v3);
}

vector vecdif0(vector v1,vector v2,vector *v3)
    /* Stores a difference of v1 and v2 to *v3 and returns *v3. If v3=NULL then
    the result vector is created anew and returned. *v3 can point to v1 or v2,
    which can be the same vectors. If dimensions of *v3 do not match then v3 is
    reallocated first.
    $A jul00; */
{
return vectordif0(v1,v2,v3);
}



    /* PRODUCTS OF MATRICES AND VECTORS */


void matprodvecplain(matrix m1,vector v2,vector res)
    /* V vektor res zapise produkt matrike m1 in vektorja v2. Funkcija ne
    kontrolira niti dimenzij niti tega, ce je prostor alociran. Vektor res ne
    sme biti isti kot v2.
    $A Igor avg00; */
{
int i,k;
double s;
for (i=1;i<=m1->d1;++i)
{
  s=0;
  for (k=1;k<=m1->d2;++k)
    s+=m1->m[i][k]*v2->v[k];
  res->v[i]=s;
}
}


vector matprodvec0(matrix m1,vector v2,vector *v3)
    /* Vrne produkt matrike m1 in vektorja v2. Ce je v3 razlicen od NULL,
    zapise rezultat v *v3 in vrne *v3. v3 je lahko tudi naslov v2, funkcija v
    tem primeru poskrbi za to, da se najprej izracuna rezultat operacije in
    sele nato prepise operand.
     OPOMBA:
     Ce je v3 razlicen od NULL, se po navadi vektor, ki ga funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze v3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor avg00; */
{
vector v=NULL,*save=NULL;
if (v3!=NULL)
{
  if (*v3==v2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce v3 kaze na
    v2: */
    save=v3;
    v3=&v;
  }
  /* Ce je v3!=NULL, se rezultat shrane v *v3: */
  /* Najprej preverimo, ce sta m1 in v2 kompatibilna z operacijo: */
  if (m1==NULL || v2==NULL)
    dispvector(v3);
  else if (m1->d2!=v2->d || m1->d1<1 || m1->d2<1)
    dispvector(v3);
  else
  {
    /* Preverimo dimenzijo *v3, ce ni ustrezna, zbrisemo *v3: */
    if (*v3!=NULL)
      if ((*v3)->d!=m1->d1)
        dispvector(v3);
    /* Ce je *v3==NULL, tvorimo *v3 s 1. dimenzijo m1: */
    if (*v3==NULL)
      *v3=getvector(m1->d1);
    /* Izvedba operacije: */
    matprodvecplain(m1,v2,*v3);
  }
  if (save!=NULL)
  {
    dispvector(save);
    *save=*v3;
  }
  return *v3; /* Ker je bil v3!=NULL, se vrne *v3. */
} else
{
  /* v3==NULL, naredi se nov vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || v2==NULL)
    return NULL;
  else if (m1->d2!=v2->d || m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    v=getvector(m1->d1);
    /* Izvedba operacije: */
    matprodvecplain(m1,v2,v);
    return v;
  }
}
}


void mattranspprodvecplain(matrix m1,vector v2,vector res)
    /* V vektor res zapise produkt transponirane matrike m1 in vektorja v2.
    Funkcija ne kontrolira niti dimenzij niti tega, ce je prostor alociran.
    Vektor res ne sme biti isti kot v2.
    $A Igor avg00; */
{
int i,k;
double s;
for (i=1;i<=m1->d2;++i)
{
  s=0;
  for (k=1;k<=m1->d1;++k)
    s+=m1->m[k][i]*v2->v[k];
  res->v[i]=s;
}
}


vector mattranspprodvec0(matrix m1,vector v2,vector *v3)
    /* Vrne produkt transponirane  matrike m1 in vektorja v2. Ce je v3 razlicen
    od NULL, zapise rezultat v *v3 in vrne *v3. v3 je lahko tudi naslov v2,
    funkcija v tem primeru poskrbi za to, da se najprej izracuna rezultat
    operacije in sele nato prepise operand.
     OPOMBA:
     Ce je v3 razlicen od NULL, se po navadi vektor, ki ga funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze v3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor avg00; */
{
vector v=NULL,*save=NULL;
if (v3!=NULL)
{
  if (*v3==v2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce v3 kaze na
    v2: */
    save=v3;
    v3=&v;
  }
  /* Ce je v3!=NULL, se rezultat shrane v *v3: */
  /* Najprej preverimo, ce sta m1 in v2 kompatibilna z operacijo: */
  if (m1==NULL || v2==NULL)
    dispvector(v3);
  else if (m1->d1!=v2->d || m1->d1<1 || m1->d2<1)
    dispvector(v3);
  else
  {
    /* Preverimo dimenzijo *v3, ce ni ustrezna, zbrisemo *v3: */
    if (*v3!=NULL)
      if ((*v3)->d!=m1->d2)
        dispvector(v3);
    /* Ce je *v3==NULL, tvorimo *v3 s 1. dimenzijo m1: */
    if (*v3==NULL)
      *v3=getvector(m1->d2);
    /* Izvedba operacije: */
    matprodvecplain(m1,v2,*v3);
  }
  if (save!=NULL)
  {
    dispvector(save);
    *save=*v3;
  }
  return *v3; /* Ker je bil v3!=NULL, se vrne *v3. */
} else
{
  /* v3==NULL, naredi se nov vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || v2==NULL)
    return NULL;
  else if (m1->d1!=v2->d || m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    v=getvector(m1->d2);
    /* Izvedba operacije: */
    matprodvecplain(m1,v2,v);
    return v;
  }
}
}



    /* SUMS AND PRODUCTS: MATRICES */



void matsumplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise vsoto matrik m1 in m2. Funkcija ne kontrolira
    niti dimenzij niti tega, ce je prostor alociran. Matrika res je lahko ista
    kot m1 ali m2, prav tako sta m1 in m2 lahko isti matriki.
    $A Igor jan00; */
{
int i,j;
for (i=1;i<=m1->d1;++i)
  for (j=1;j<=m1->d2;++j)
    res->m[i][j]=m1->m[i][j]+m2->m[i][j];
}


void matdifplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise razliko matrik m1 in m2. Funkcija ne kontrolira
    niti dimenzij niti tega, ce je prostor alociran. Matrika res je lahko ista
    kot m1 ali m2, prav tako sta m1 in m2 lahko isti matriki.
    $A Igor jan00; */
{
int i,j;
for (i=1;i<=m1->d1;++i)
  for (j=1;j<=m1->d2;++j)
    res->m[i][j]=m1->m[i][j]-m2->m[i][j];
}

matrix matsum0(matrix m1,matrix m2,matrix *m3)
    /* Stores a sum of m1 and m1 to *m3 and returns *m3. If m3=NULL then the
    sum is created anew and returned. *m3 can point to m1 or m2, which can be
    the same matrices. If dimensions of *m3 do not match then *m3 is reallocated
    first.
    $A jul00; */
{
return matrixsum0(m1,m2,m3);
}


matrix matdif0(matrix m1,matrix m2,matrix *m3)
    /* Stores a difference of m1 and m1 to *m3 and returns *m3. If m3=NULL then
    the difference is created anew and returned. *m3 can point to m1 or m2,
    which can be the same matrices. If dimensions of *m3 do not match then *m3
    is reallocated first.
    $A jul00; */
{
return matrixdif0(m1,m2,m3);
}



void matprodplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise produkt matrik m1 in m2. Funkcija ne kontrolira
    niti dimenzij niti tega, ce je prostor alociran. Matrika res ne sme biti
    ista kot m1 ali m2, m1 in m2 pa sta lahko isti.
    $A Igor jan00; */
{
int i,j,k;
double s;
for (i=1;i<=m1->d1;++i)
  for (j=1;j<=m2->d2;++j)
  {
    s=0;
    for (k=1;k<=m1->d2;++k)
      s+=m1->m[i][k]*m2->m[k][j];
    res->m[i][j]=s;
  }
}

matrix matprod0(matrix m1,matrix m2,matrix *m3)
    /* Vrne produkt matrik m1 in m2. Ce je m3 razlicen od NULL, zapise rezultat
    v *m3 in vrne *m3. m3 je lahko tudi naslov m1 ali m2, funkcija v tem
    primeru poskrbi za to, da se najprej izracuna rezultat operacije in sele
    nato prepise operand.
     OPOMBA:
     Ce je m3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze m3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (m3!=NULL)
{
  if (*m3==m1 || *m3==m2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce m3 kaze na
    m1 ali m2: */
    save=m3;
    m3=&m;
  }
  /* Ce je m3!=NULL, se rezultat shrane v *m3: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni z operacijo: */
  if (m1==NULL || m2==NULL)
    dispmatrix(m3);
  else if (m1->d2!=m2->d1 || m1->d1<1 || m1->d2<1 || m2->d2<1)
    dispmatrix(m3);
  else
  {
    /* Preverimo dimenzijo *m3, ce ni ustrezna, zbrisemo *m3: */
    if (*m3!=NULL)
      if ((*m3)->d1!=m1->d1 || (*m3)->d2!=m2->d2)
        dispmatrix(m3);
    /* Ce je *m3==NULL, tvorimo *m3 z dimenzijama m1: */
    if (*m3==NULL)
      *m3=getmatrix(m1->d1,m2->d2);
    /* Izvedba operacije: */
    matprodplain(m1,m2,*m3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*m3;
  }
  return *m3; /* Ker je bil m3!=NULL, se vrne *m3. */
} else
{
  /* m3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || m2==NULL)
    return NULL;
  else if (m1->d2!=m2->d1 || m1->d1<1 || m1->d2<1 || m2->d2<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m2->d2);
    /* Izvedba operacije: */
    matprodplain(m1,m2,m);
    return m;
  }
}
}


void mattranspprodmatplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise produkt transponirane matrike m1 in matrike m2.
    Funkcija ne kontrolira niti dimenzij niti tega, ce je prostor alociran.
    Matrika res ne sme biti ista kot m1 ali m2, m1 in m2 pa sta lahko isti.
    $A Igor jan00; */
{
int i,j,k;
double s;
for (i=1;i<=m1->d2;++i)
  for (j=1;j<=m2->d2;++j)
  {
    s=0;
    for (k=1;k<=m1->d1;++k)
      s+=m1->m[k][i]*m2->m[k][j];
    res->m[i][j]=s;
  }
}

matrix mattranspprodmat0(matrix m1,matrix m2,matrix *m3)
    /* Vrne produkt transponirane matrike m1 in matrike m2. Ce je m3 razlicen
    od NULL, zapise rezultat v *m3 in vrne *m3. m3 je lahko tudi naslov m1 ali
    m2, funkcija v tem primeru poskrbi za to, da se najprej izracuna rezultat
    operacije in sele nato prepise operand. Ce matriki nista kompatibilni za
    operacijo, je rezultat NULL.
     OPOMBA:
     Ce je m3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze m3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (m3!=NULL)
{
  if (*m3==m1 || *m3==m2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce m3 kaze na
    m1 ali m2: */
    save=m3;
    m3=&m;
  }
  /* Ce je m3!=NULL, se rezultat shrane v *m3: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni z operacijo: */
  if (m1==NULL || m2==NULL)
    dispmatrix(m3);
  else if (m1->d1!=m2->d1 || m1->d1<1 || m1->d2<1 || m2->d2<1)
    dispmatrix(m3);
  else
  {
    /* Preverimo dimenzijo *m3, ce ni ustrezna, zbrisemo *m3: */
    if (*m3!=NULL)
      if ((*m3)->d1!=m1->d2 || (*m3)->d2!=m2->d2)
        dispmatrix(m3);
    /* Ce je *m3==NULL, tvorimo *m3 z dimenzijama m1: */
    if (*m3==NULL)
      *m3=getmatrix(m1->d2,m2->d2);
    /* Izvedba operacije: */
    mattranspprodmatplain(m1,m2,*m3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*m3;
  }
  return *m3; /* Ker je bil m3!=NULL, se vrne *m3. */
} else
{
  /* m3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || m2==NULL)
    return NULL;
  else if (m1->d1!=m2->d1 || m1->d1<1 || m1->d2<1 || m2->d2<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d2,m2->d2);
    /* Izvedba operacije: */
    mattranspprodmatplain(m1,m2,m);
    return m;
  }
}
}




void matprodmattranspplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise produkt matrike m1 in transponirane matrike m2.
    Funkcija ne kontrolira niti dimenzij niti tega, ce je prostor alociran.
    Matrika res ne sme biti ista kot m1 ali m2, m1 in m2 pa sta lahko isti.
    $A Igor jan00; */
{
int i,j,k;
double s;
for (i=1;i<=m1->d1;++i)
  for (j=1;j<=m2->d1;++j)
  {
    s=0;
    for (k=1;k<=m1->d2;++k)
      s+=m1->m[i][k]*m2->m[j][k];
    res->m[i][j]=s;
  }
}

matrix matprodmattransp0(matrix m1,matrix m2,matrix *m3)
    /* Vrne produkt matrike m1 in transponirane matrike m2. Ce je m3 razlicna
    od NULL, zapise rezultat v *m3 in vrne *m3. m3 je lahko tudi naslov m1 ali
    m2, funkcija v tem primeru poskrbi za to, da se najprej izracuna rezultat
    operacije in sele nato prepise operand.
     OPOMBA:
     Ce je m3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze m3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (m3!=NULL)
{
  if (*m3==m1 || *m3==m2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce m3 kaze na
    m1 ali m2: */
    save=m3;
    m3=&m;
  }
  /* Ce je m3!=NULL, se rezultat shrane v *m3: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni z operacijo: */
  if (m1==NULL || m2==NULL)
    dispmatrix(m3);
  else if (m1->d2!=m2->d2 || m1->d1<1 || m1->d2<1 || m2->d1<1)
    dispmatrix(m3);
  else
  {
    /* Preverimo dimenzijo *m3, ce ni ustrezna, zbrisemo *m3: */
    if (*m3!=NULL)
      if ((*m3)->d1!=m1->d1 || (*m3)->d2!=m2->d1)
        dispmatrix(m3);
    /* Ce je *m3==NULL, tvorimo *m3 z dimenzijama m1: */
    if (*m3==NULL)
      *m3=getmatrix(m1->d1,m2->d1);
    /* Izvedba operacije: */
    matprodmattranspplain(m1,m2,*m3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*m3;
  }
  return *m3; /* Ker je bil m3!=NULL, se vrne *m3. */
} else
{
  /* m3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || m2==NULL)
    return NULL;
  else if (m1->d2!=m2->d2 || m1->d1<1 || m1->d2<1 || m2->d1<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m2->d1);
    /* Izvedba operacije: */
    matprodmattranspplain(m1,m2,m);
    return m;
  }
}
}


void mattranspprodmattranspplain(matrix m1,matrix m2,matrix res)
    /* V matriko res zapise produkt transponirane matrike m1 in transponirane
    matrike m2. Funkcija ne kontrolira niti dimenzij niti tega, ce je prostor
    alociran. Matrika res ne sme biti ista kot m1 ali m2, m1 in m2 pa sta
    lahko isti.
    $A Igor jan00; */
{
int i,j,k;
double s;
for (i=1;i<=m1->d2;++i)
  for (j=1;j<=m2->d1;++j)
  {
    s=0;
    for (k=1;k<=m1->d1;++k)
      s+=m1->m[k][i]*m2->m[j][k];
    res->m[i][j]=s;
  }
}

matrix mattranspprodmattransp0(matrix m1,matrix m2,matrix *m3)
    /* Vrne produkt transponirane matrike m1 in transponirane matrike m2. Ce je
    m3 razlicna od NULL, zapise rezultat v *m3 in vrne *m3. m3 je lahko tudi
    naslov m1 ali m2, funkcija v tem primeru poskrbi za to, da se najprej
    izracuna rezultat operacije in sele nato prepise operand. Ce matriki nista
    kompatibilni za operacijo, je rezultat NULL.
     OPOMBA:
     Ce je m3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze m3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (m3!=NULL)
{
  if (*m3==m1 || *m3==m2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce m3 kaze na
    m1 ali m2: */
    save=m3;
    m3=&m;
  }
  /* Ce je m3!=NULL, se rezultat shrane v *m3: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni z operacijo: */
  if (m1==NULL || m2==NULL)
    dispmatrix(m3);
  else if (m1->d1!=m2->d2 || m1->d1<1 || m1->d2<1 || m2->d1<1)
    dispmatrix(m3);
  else
  {
    /* Preverimo dimenzijo *m3, ce ni ustrezna, zbrisemo *m3: */
    if (*m3!=NULL)
      if ((*m3)->d1!=m1->d2 || (*m3)->d2!=m2->d1)
        dispmatrix(m3);
    /* Ce je *m3==NULL, tvorimo *m3 z dimenzijama m1: */
    if (*m3==NULL)
      *m3=getmatrix(m1->d2,m2->d1);
    /* Izvedba operacije: */
    mattranspprodmattranspplain(m1,m2,*m3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*m3;
  }
  return *m3; /* Ker je bil m3!=NULL, se vrne *m3. */
} else
{
  /* m3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta m1 in m2 kompatibilni za operacijo: */
  if (m1==NULL || m2==NULL)
    return NULL;
  else if (m1->d1!=m2->d2 || m1->d1<1 || m1->d2<1 || m2->d1<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d2,m2->d1);
    /* Izvedba operacije: */
    mattranspprodmattranspplain(m1,m2,m);
    return m;
  }
}
}




void mattranspplain(matrix m,matrix res)
    /* Elementi matrike res postanejo elementi transponirane matrike m.
    Funkcija ne preverja dimenzij in obstoja matrik, zato morajo biti dimenzije
    kompatibilne. Ce j m kvadratna matrika, je lahko res ista matrika kot m.
    $A Igor jan00; */
{
int i,j;
if (res->d1==res->d2)
{
  double x;
  /* Kvadratne matrike: */
  for (i=1;i<=res->d1;++i)
  {
    res->m[i][i]=m->m[i][i];  /* Diagonal. element */
    for (j=1;j<i;++j)
    {
      /* Izvendiagonalni elementi; Ce sta m in res isti matriki, vmesna
      spremenljivka x prepreci, da bi se poddiagonalni element original.
      matrike prepisal z ustreznim naddiagonalnim, preden je prirejen: */
      x=m->m[i][j];
      res->m[i][j]=m->m[j][i];
      res->m[j][i]=x;
    }
  }
} else
{
  /* Nekvadratne matrike: */
  for (i=1;i<=res->d1;++i)
    for (j=1;j<=res->d2;++j)
      res->m[i][j]=m->m[j][i];
}
}


matrix mattransp0(matrix m1,matrix *res)
    /* Vrne transponirano matriko m1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. res je lahko tudi naslov m1, funkcija v tem
    primeru poskrbi za to, da se najprej izracuna rezultat operacije in sele
    nato prepise operand.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (res!=NULL)
{
  if (*res==m1 && m1!=NULL)
    if (m1->d1!=m1->d2)
    {
      /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce res kaze na
      m1 in m1 ni kvadratna matrika: */
      save=res;
      res=&m;
    }
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (m1->d1<1 || m1->d2<1)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=m1->d2 || (*res)->d2!=m1->d1)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(m1->d2,m1->d1);
    /* Izvedba operacije: */
    mattranspplain(m1,*res);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*res;
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1)
    return NULL;
  else
  {
    m=getmatrix(m1->d2,m1->d1);
    /* Izvedba operacije: */
    mattranspplain(m1,m);
    return m;
  }
}
}



            /***************************/
            /*                         */
            /*  INVERSION OF MATRICES  */
            /*                         */
            /***************************/



static void matinvplain1(matrix mat,matrix res)
    /* Kvadratna matrika res postane inverzna matrika matrike mat, ce je m 
    obrnljiva. Funkcija ne preverja dimenzij matrik, zato morajo biti te
    kompatibilne.
    - INVERZ PO TOMAZU SUSTERJU
    $A Igor jan00; */
{
int i,j;
int *order=NULL;
vector scale=NULL;
matrix coef=NULL;
coef=getmatrix(mat->d1,mat->d2);
for (i=1;i<=mat->d1;++i)
  for (j=1;j<=mat->d2;++j)
    coef->m[i][j]=mat->m[i][j];
order=malloc((1+mat->d1)*sizeof(*order));
scale=getvector(mat->d1);
luordermat(scale,coef,order);
ludecompmat(scale,coef,order);
luinvmat(coef,res,order);
free(order);
dispmatrix(&coef);
dispvector(&scale);
}


matrix matinv1(matrix m1,matrix *res)
    /* Vrne inverz matrike m1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. res je lahko tudi naslov m1, funkcija v tem
    primeru poskrbi za to, da se najprej izracuna rezultat operacije in sele
    nato prepise operand.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    - INVERZ PO TOMAZU SUSTERJU
    $A Igor jan00; */
{
matrix m=NULL,*save=NULL;
if (res!=NULL)
{
  if (*res==m1)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce res kaze na
    m1: */
    save=res;
    res=&m;
  }
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=m1->d1 || (*res)->d2!=m1->d2)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    matinvplain1(m1,*res);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*res;
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    matinvplain1(m1,m);
    return m;
  }
}
}


void matinvplain(matrix mat,matrix res)
    /* Kvadratna matrika res postane inverzna matrika matrike mat, ce je m 
    obrnljiva. Funkcija ne preverja dimenzij matrik, zato morajo biti te
    kompatibilne.
    $A Igor jan00; */
{
matinvplain1(mat,res);
}

matrix matinv0(matrix m1,matrix *res)
    /* Vrne inverz matrike m1. Ce je res razlicen od NULL, zapise
    rezultat v *res in vrne *res. res je lahko tudi naslov m1, funkcija v tem
    primeru poskrbi za to, da se najprej izracuna rezultat operacije in sele
    nato prepise operand.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
return matinv1(m1,res);
}




            /********************************/
            /*                              */
            /*  DECOMPOSITIONS OF MATRICES  */
            /*                              */
            /********************************/



    /* LLT (Choleski) DECOMPOSITION (symmetric positive definite matrices) */

int LLTdecomptolplain(matrix m1,matrix res,double smalltol)
    /* Izvede Choleskijevo dekompozicijo matrike m1 in shrane rezultat v res.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 pozitivno definitna)
    ali negativno vrednost, ce operacija ni bila uspesna. V tem primeru vrne
    minus stevilko vrstice, kjer se je pojavil negativni diagonalni element,
    ali -m1->d-1, ce je kvadrat razmerja med min. in maks. elemantom manj kot
    smalltol.
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan diagonalni element ali deljitelj enak 0 (obicajno je kar 0!).
    Matrika res je lahko ista kot m1, v tem primeru funkcija zapise v m1
    spodnjetrikotne elemente dekompozicije ter nicle nad diagonalo.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira z zgornjim trikotnikom matrike.
    $A Igor jan00; */
{
int i,j,k,ret=0;
double s,mindiag,maxdiag;
smalltol=fabs(smalltol);
for (i=1;i<=m1->d1;++i)
{
  /* Diagonalni element: */
  s=m1->m[i][i];
  for (k=1;k<i;++k)
    s-=res->m[i][k]*res->m[i][k];
  if (s<=0)
  {
    printf("\n\nLLT - Check 0: l[%i,%i]<=0\n\n",i,i);
    return -i;
  }
  if (i==1)
    maxdiag=mindiag=fabs(s);
  else
  {
    maxdiag=maxval(maxdiag,fabs(s));
    mindiag=minval(mindiag,fabs(s));
  }
  res->m[i][i]=sqrt(s);
  /* Izvendiagonalni elementi: */
  for (j=i+1;j<=m1->d1;++j)
  {
    s=m1->m[i][j];
    for (k=1;k<i;++k)
      s-=res->m[i][k]*res->m[j][k];
    res->m[j][i]=s/res->m[i][i];
    res->m[i][j]=0;
  }
}
if (mindiag/maxdiag<smalltol)
{
  printf("n\nLLT - Check 1: (min. diag.)/(max.diag.) < %g\n\n",smalltol);
  ret=-m1->d1-1;
}
return ret;
}



int LLTdecomptolplainrows(matrix m1,matrix res,double smalltol)
    /* Izvede Choleskijevo dekompozicijo matrike m1 in shrane rezultat v res.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 pozitivno definitna)
    ali negativno vrednost, ce operacija ni bila uspesna.
      Dekompozicija se izvede po vrsticah, v nasprotju s funkcijo
    LLTdecomptolplain, ki jo izvede po stolpcih. V splosnem naj se uporablja
    LLTdecomptolplain!
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan diagonalni element ali deljitelj enak 0 (obicajno je kar 0!).
    Matrika res in d je lahko ista kot m1, v tem primeru funkcija zapise
    v m1 spodnjetrikotne elemente res ter nicle nad diagonalo.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira s spodnjim trikotnikom matrike.
    $A Igor jan00; */
{
int i,j,k,ret=0;
double s,mindiag,maxdiag;
smalltol=fabs(smalltol);
for (i=1;i<=m1->d1;++i)
{
  /* Izvendiagonalni elementi: */
  for (j=1;j<i;++j)
  {
    s=m1->m[i][j];
    for (k=1;k<j;++k)
      s-=res->m[i][k]*res->m[j][k];
    res->m[i][j]=s/res->m[j][j];
    res->m[j][i]=0;
  }
  /* Diagonalni element: */
  s=m1->m[i][i];
  for (k=1;k<i;++k)
    s-=res->m[i][k]*res->m[i][k];
  if (s<=0)
  {
    printf("\n\nLLT - Check 0: l[%i,%i]<=0\n\n",i,i);
    return -i;
  }
  if (i==1)
    maxdiag=mindiag=fabs(s);
  else
  {
    maxdiag=maxval(maxdiag,fabs(s));
    mindiag=minval(mindiag,fabs(s));
  }
  res->m[i][i]=sqrt(s);
}
if (mindiag/maxdiag<smalltol)
{
  printf("n\nLLT - Check 1: (min. diag.)/(max.diag.) < %g\n\n",smalltol);
  ret=-m1->d1-1;
}
return ret;
}


matrix LLTdecomptol0(matrix m1,matrix *res,double smalltol)
    /* Vrne LLT (Choleskijevo) dekompozicijo matrike m1. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. res je lahko tudi naslov m1.
    Funkcija vrne NULL, ce dekompozicija  ni bila uspesna. Funkcija ne cekira,
    ce je matrika m1 simetricna, temvec operacijo enostavno izvede na zgornjem
    trikotnik matrike.
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan diagonalni element ali deljitelj enak 0 (obicajno je kar 0!).
    Matrika res in d je lahko ista kot m1, v tem primeru funkcija zapise
    v m1 spodnjetrikotne elemente res ter nicle nad diagonalo.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
matrix m=NULL;
if (res!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
    dispmatrix(res);
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    dispmatrix(res);
  else
  {
    /* Preverimo dimenzijo *res, ce ni ustrezna, zbrisemo *res: */
    if (*res!=NULL)
      if ((*res)->d1!=m1->d1 || (*res)->d2!=m1->d2)
        dispmatrix(res);
    /* Ce je *res==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*res==NULL)
      *res=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    if (LLTdecomptolplain(m1,*res,smalltol))
      dispmatrix(res); /* operacija je bila neuspesna */
  }
  return *res; /* Ker je bil res!=NULL, se vrne *res. */
} else
{
  /* res==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    if (LLTdecomptolplain(m1,m,smalltol))
      dispmatrix(&m);
    return m;
  }
}
}

int LLTdecompplain(matrix m1,matrix res)
    /* Izvede Choleskijevo dekompozicijo matrike m1 in shrane rezultat v res.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 pozitivno definitna)
    ali -1, ce operacija ni bila uspesna.
    Matrika res in d je lahko ista kot m1, v tem primeru funkcija zapise
    v m1 spodnjetrikotne elemente res ter nicle nad diagonalo.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira z zgornjim trikotnikom matrike.
    $A Igor jan00; */
{
return LLTdecomptolplain(m1,res,0);
}


matrix LLTdecomp0(matrix m1,matrix *res)
    /* Vrne LLT (Choleskijevo) dekompozicijo matrike m1. Ce je res razlicen od
    NULL, zapise rezultat v *res in vrne *res. res je lahko tudi naslov m1.
    Funkcija vrne NULL, ce dekompozicija  ni bila uspesna. Funkcija ne cekira,
    ce je matrika m1 simetricna, temvec operacijo enostavno izvede na zgornjem
    trikotnik matrike.
     OPOMBA:
     Ce je res razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze res (ker imamo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor jan00; */
{
return LLTdecomptol0(m1,res,0);
}



    /*  LDLT DECOMPOSITION (symmetric matrices) */



int LDLTdecomptolplain(matrix m1,matrix res,matrix d,double smalltol)
    /* Izvede LDLT dekompozicijo matrike m1 in shrane rezultat v l in d.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 obrnljiva), ali
    negativno vrednost, ce operacija ni bila uspesna in sicer -i, ce je
    diagonalni element i tocno 0, ali -m1->d1-1, ce je razmerje med absolutno
    najmanjsim in najvecjim izracunanim diagonalnim elementom manj kot
    smalltol (v tem primeru se dekompozicija sicer izracuna do konca, le da je
    treba racunati z vecjim vplivom numericnih napak).
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    razmerje med absolutno najmanjsim in najvecjim diagonalnim elementom zelo
    malo (slaba pogojenost - velikokrat damo za toleranco kar 0!).
    Matriki res in d sta lahko isti kot m1, v tem primeru funkcija zapise
    v m1 diegonalne elemente d in poddiagonalne elemente matrike res, nad
    diagonalo pa zapise nicle.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira z zgornjim trikotnikom matrike.
    $A Igor jan00; */
{
int i,j,k,ret=0,dim1,dim2;
double s,maxdiag,mindiag;
double *line_m1_i,*line_res_i,*line_res_j,*diag;
vector vdiag=NULL;
dim1=m1->d1;
dim2=m1->d2;
if (dim1>20)
{
  /* Optimised version by using auxiliary pointers: */
  vdiag=getvector(dim1);
  diag=vdiag->v;
  for (i=1;i<=dim1;++i)
  {
    line_m1_i=m1->m[i];
    line_res_i=res->m[i];
    /* Diagonalni element: */
    s=line_m1_i[i];
    for (k=1;k<i;++k)
      s-=line_res_i[k]*line_res_i[k]*diag[k];
    line_res_i[i]=1;
    diag[i]=s; /* sqrt(s); */
    if (i==1)
      maxdiag=mindiag=fabs(s);
    else
    {
      maxdiag=maxval(maxdiag,fabs(s));
      mindiag=minval(mindiag,fabs(s));
    }
    if (s==0)
    {
      printf("\n\nCheck 0: diag[%i,%i]=0\n\n",i,i);
      return -i;
    }
    /* Izveniagonalni elementi: */
    for (j=i+1;j<=dim1;++j)
    {
      line_res_j=res->m[j];
      s=line_m1_i[j];
      for (k=1;k<i;++k)
        s-=line_res_i[k]*line_res_j[k]*diag[k];
      d->m[i][j]=d->m[j][i]=0;
      line_res_j[i]=s/diag[i];
    }
  }
  for (i=1;i<=dim1;++i)
    d->m[i][i]=diag[i];
  dispvector(&vdiag);
} else
  {
    /* Original version, components of matrices and vectors are accessed
    directly: */
  for (i=1;i<=m1->d1;++i)
  {
    /* Diagonalni element: */
    s=m1->m[i][i];
    for (k=1;k<i;++k)
      s-=res->m[i][k]*res->m[i][k]*d->m[k][k];
    res->m[i][i]=1;
    d->m[i][i]=s; /* sqrt(s); */
    if (i==1)
      maxdiag=mindiag=fabs(s);
    else
    {
      maxdiag=maxval(maxdiag,fabs(s));
      mindiag=minval(mindiag,fabs(s));
    }
    if (s==0)
    {
      printf("\n\nCheck 0: diag[%i,%i]=0\n\n",i,i);
      return -i;
    }
    /* Izveniagonalni elementi: */
    for (j=i+1;j<=m1->d1;++j)
    {
      s=m1->m[i][j];
      for (k=1;k<i;++k)
        s-=res->m[i][k]*res->m[j][k]*d->m[k][k];
      d->m[i][j]=d->m[j][i]=0;
      res->m[j][i]=s/d->m[i][i];
    }
  }
}
if (mindiag/maxdiag<smalltol)
{
  printf("\n\nLDLT - Check 1: (min. diag./max.diag.)<%g\n\n",i,i,smalltol);
  ret=-m1->d1-1;
}
return ret;
}


int LDLTdecomptolplainrows(matrix m1,matrix res,matrix d,double smalltol)
    /* Izvede LDLT dekompozicijo matrike m1 in shrane rezultat v l in d.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 obrnljiva) ali
    negativno stevilo (podobno kot LDLTdecomptolplain), ce operacija ni bila
    uspesna.
    Elementi matrike L so v nasprotju s funkcijo LDLTdecomptolplain izracunani
    po vrsticah. Uporablja naj se kar funkcija LDLTdecomptolplain!
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan deljitelj enak 0 (obicajno je ta kar 0!).
    Matriki res in d sta lahko isti kot m1, v tem primeru funkcija zapise
    v m1 diegonalne elemente d in poddiagonalne elemente matrike res, nad
    diagonalo pa zapise nicle.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira z zgornjim trikotnikom matrike.
    $A Igor jan00; */
{
int i,j,k,ret=0;
double s,mindiag,maxdiag;
for (i=1;i<=m1->d1;++i)
{
  /* Izveniagonalni elementi: */
  for (j=1;j<i;++j)
  {
    s=m1->m[i][j];
    for (k=1;k<j;++k)
      s-=res->m[i][k]*res->m[j][k]*d->m[k][k];
    /*
    if (fabs(d->m[j][j])<=smalltol)
    {
      printf("\n\nCheck 1: l[%i,%i]<tol\n\n",i,i);
      return -1;
    }
    */
    d->m[i][j]=d->m[j][i]=0;
    res->m[i][j]=s/d->m[j][j];
  }
  /* Diagonalni element: */
  s=m1->m[i][i];
  for (k=1;k<i;++k)
    s-=res->m[i][k]*res->m[i][k]*d->m[k][k];
  res->m[i][i]=1;
  d->m[i][i]=s; /* sqrt(s); */
  if (i==1)
    maxdiag=mindiag=fabs(s);
  else
  {
    maxdiag=maxval(maxdiag,fabs(s));
    mindiag=minval(mindiag,fabs(s));
  }
  if (s==0)
  {
    printf("\n\nLDLT - Check 0: l[%i,%i]=0\n\n",i,i);
    return -i;
  }
}
if (mindiag/maxdiag<smalltol)
{
  printf("\n\nLDLT - Check 1: (min. diag./max.diag.)<%g\n\n",i,i,smalltol);
  ret=-m1->d1-1;
}
return ret;
}


matrix LDLTdecomptol0(matrix m1,matrix *resL,matrix *resD,double smalltol)
    /* Vrne LDLT dekompozicijo matrike m1. Spodnje trikotno matriko zapise v
    matriko *resL , ki jo tudi vrne, diagonalno pa v resD. resL ali resD sta
    lahko tudi naslova m1 ali naslova iste matrike.
    Funkcija vrne NULL, ce dekompozicija  ni bila uspesna. Funkcija ne cekira,
    ce je matrika m1 simetricna, temvec operacijo enostavno izvede na zgornjem
    trikotnik matrike.
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan deljitelj enak 0 (obicajno je kar 0!).
     OPOMBA:
     resL in resD sta lahko tudi NULL.
    $A Igor jan00; */
{
matrix m=NULL;
if (resD==NULL)
  resD=resL;
else if (resL==NULL)
  resL=resD;
if (resL!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
  {
    dispmatrix(resL);
    dispmatrix(resD);
  }
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
  {
    dispmatrix(resL);
    dispmatrix(resD);
  }
  else
  {
    /* Preverimo dimenzijo *resL in *resD, ce ni ustrezna, ju zbrisemo: */
    if (*resL!=NULL)
      if ((*resL)->d1!=m1->d1 || (*resL)->d2!=m1->d2)
        dispmatrix(resL);
    /* Ce je *resL==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*resL==NULL)
      *resL=getmatrix(m1->d1,m1->d2);
    if (*resD!=NULL)
      if ((*resD)->d1!=m1->d1 || (*resD)->d2!=m1->d2)
        dispmatrix(resD);
    /* Ce je *resD==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*resD==NULL)
      *resD=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    if (LDLTdecomptolplain(m1,*resL,*resD,smalltol))
    {
      /* Operacija je bila neuspesna: */
      dispmatrix(resL); 
      dispmatrix(resD);
    }
  }
  return *resL;
} else
{
  /* resD=resL=NULL, naredi se nova matrika, v katero se zapise rezultat
  operacije, vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    if (LDLTdecomptolplain(m1,m,m,smalltol))
      dispmatrix(&m);
    return m;
  }
}
}


int LDLTdecompplain(matrix m1,matrix res,matrix d)
    /* Izvede LDLT dekompozicijo matrike m1 in shrane rezultat v l in d.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 obrnljiva) ali -1,
    ce operacija ni bila uspesna.
    Matriki res in d sta lahko isti kot m1, v tem primeru funkcija zapise
    v m1 diegonalne elemente d in poddiagonalne elemente matrike res, nad
    diagonalo pa zapise nicle.
     POZOR:
     Funkcija tudi ne kontrolira, ce je matrika m1 simetricna, temvec enostavno
    operira z zgornjim trikotnikom matrike.
    $A Igor jan00; */
{
return LDLTdecomptolplain(m1,res,d,0);
}

matrix LDLTdecomp0(matrix m1,matrix *resL,matrix *resD)
    /* Vrne LDLT dekompozicijo matrike m1. Spodnje trikotno matriko zapise v
    matriko *resL , ki jo tudi vrne, diagonalno pa v resD. resL ali resD sta
    lahko tudi naslova m1 ali naslova iste matrike.
    Funkcija vrne NULL, ce dekompozicija  ni bila uspesna. Funkcija ne cekira,
    ce je matrika m1 simetricna, temvec operacijo enostavno izvede na ZGORNJEM
    TRIKOTNIKU matrike.
     OPOMBA:
     resL in resD sta lahko tudi NULL.
    $A Igor jan00; */
{
return LDLTdecomptol0(m1,resL,resD,0);
}



int LUdecomptolplainold(matrix m1,matrix ml,matrix mu,indtab ind,
                     int *parity,double smalltol)
    /* Izvede LU dekompozicijo matrike m1 in shrane rezultat v l in d.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 obrnljiva) ali -1,
    ce operacija ni bila uspesna.
    indtab je lahko NULL, v tem primeru se dekompozicija izvede brez
    pivotiranja, vendar to NI PRIPOROCLJIVO, ker lahko dobimo 0 na diagonali,
    ceprav matrika ni singularna. parity je lahko NULL, v tem primeru se ne
    zabelezi, ce je bilo zamenjav liho stevilo, in se ne more racunati
    determinanta matrike.
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan deljitelj enak 0 (obicajno je ta kar 0!).
    Matriki res in d sta lahko isti kot m1, v tem primeru funkcija zapise
    v m1 diegonalne elemente d in poddiagonalne elemente matrike res, nad
    diagonalo pa zapise nicle.
      parity je lahko enak NULL, v tem primeru se ne zabelezi, ali je stevilo
    zamenjav bilo liho ali sodo (to se tako ali tako rabi samo pri racunanju
    determinante). Tudi indtab je naceloma lahko NULL, vendat to NI
    PRIPOROCLJIVO! V tem primeru namrec funkcija ne izvaja pivotiranja in se
    lahko zgodi, da ima matriko za singularno, ceprav ni. Drugace funkcija
    izvaja implicitno pivotiranje.
    Ref. Num. Rec., p.p.43-49.
    $A Igor avg00; */
{
int ret=0;
int i,j,k,n,imax;
double x,s,val;
vector v=NULL;
n=m1->d1;
if (parity!=NULL)
  *parity=1;
if (ind!=NULL)
{
  /* Mozne so zamenjave vrstic, zato najprej poskrbimo, da se ohrani originalna
  matrika m1, ce je tako misljeno (torej, ce nobena od matrik ml in mu ni ista
  kot m1); ohranitev originalnih vrednosti dosezemo s tem, da m1 skopiramo v
  ml in uporabljamo to kopijo kot m1: */
  if (m1!=ml && m1!=mu)
  {
    for (i=1;i<=m1->d1;++i)
      for (j=1;j<=m1->d2;++j)
        ml->m[i][j]=m1->m[i][j];
    m1=ml;
  }
  v=getvector(n);
  /* Izracun skalirnih faktorjev za izbiro najboljsega pivota: */
  for (i=1;i<=n;++i)
  {
    val=0;
    for (j=1;j<=n;++j)
      if ((x=fabs(m1->m[i][j]))>val)
        val=x;
    if (val==0) /* V i. vrstici ni nenicelnega elementa, matrika je singularna */
    {
      dispvector(&v);
      return -1;
    } else
      v->v[i]=1/val;
  }
}
for (j=1;j<=n;++j) /* iteracija po stolpcih */
{
  /* Izracun elementov nad diagonalo: */
  for (i=1;i<j;++i)
  {
    s=m1->m[i][j];
    for (k=1;k<i;++k)
      s-=ml->m[i][k]*mu->m[k][j];
    mu->m[i][j]=s;
  }
  /* Izracun elementov na in pod diagonalo in pomnjenje vrstice z relativno 
  najvecjim pivotom: */
  val=0; /* najvecji primerjalni faktor za pivot */
  imax=j; /* vrstica z najboljsim pivotom */
  for (i=j;i<=n;++i)
  {
    s=m1->m[i][j];
    for (k=1;k<j;++k)
      s-=ml->m[i][k]*mu->m[k][j];
    /* Prireditev izracunane vrednosti ustreznemu elementu ustrezne matrike: */
    if (i==j)
    {
      ml->m[i][j]=1;
      mu->m[i][j]=s;
    } else
      ml->m[i][j]=s;
    if (v!=NULL)
    {
      if ((x=v->v[i]*fabs(s)) >= val)
      {
        /* Nasli smo boljsi pivot od prejsnjega najboljsega */
        val=x;
        imax=i;
      }
    }
  }  /* iteracija po vrsticah - elementi na in pod diagonalo */
  /* Zamenjava vrstic, ce je to potrebno: */
  if (v!=NULL)
  {
    ind->t[j]=imax; /* indeks v tabeli, ki belezi zamenjave vrstic */
    if (imax!=j) /* zamenjava vrstic je potrebna */
    {
      /* printf("INTERCHANGING rows %i and %i.\n",j,imax); */
      /* Zamenjava vrstic j in imax: */
      for (k=1;k<j;++k)
      {
        x=ml->m[j][k];
        ml->m[j][k]=ml->m[imax][k];
        ml->m[imax][k]=x;
      }
      x=mu->m[j][j];
      mu->m[j][j]=ml->m[imax][j];
      ml->m[imax][j]=x;
      for (k=j+1;k<=n;++k)
      {
        x=m1->m[j][k];
        m1->m[j][k]=m1->m[imax][k];
        m1->m[imax][k]=x;
      }
      /* Zaradi zamenjave se spremeni parnost, zamenjati je potrebno tudi
      skalirni faktor vrstice imax: */
      if (parity!=NULL)
        *parity=-*parity;
      v->v[imax]=v->v[j];  /* j. elementa ne rabimo vec */
    }
  }
  
  if (fabs(mu->m[j][j])<=smalltol)
  {
    /* Zabelezen je nicti pivot, matrika je singularna */
    
  }
  /* Nazadnje se deljenje s pivotom: */
  if (j<n)
  {
    x=1/mu->m[j][j];
    for (i=j+1;i<=n;++i)
      ml->m[i][j]*=x;
  }
} /* iteracija po stolpcih */
/* Ce se zgornje in spodnje trikotna matrika ne shranita v isto matriko,
postavimo naddiagonalne elemente pri ml in poddiagonalne pri mu na 0: */

if (ml!=mu)
  for (i=1;i<=n;++i)
    for (j=i+1;j<=n;++j)
      ml->m[i][j]=mu->m[j][i]=0;
dispvector(&v);
return ret;
}







int LUdecomptolplain(matrix m1,matrix ml,matrix mu,indtab ind,
                     int *parity,double smalltol)
    /* Izvede LU dekompozicijo matrike m1 in shrane rezultat v l in d.
    Funkcija sama ne kontrolira kompatibilnosti dimenzij, zato morajo biti
    dimenzije argumentov ze kompatibilne. Funkcija vrne 0, ce je lahko
    dekompozicijo uspesno opravila (t.j. ce je matrika m1 obrnljiva) ali -1,
    ce operacija ni bila uspesna.
    indtab je lahko NULL, v tem primeru se dekompozicija izvede brez
    pivotiranja, vendar to NI PRIPOROCLJIVO, ker lahko dobimo 0 na diagonali,
    ceprav matrika ni singularna. parity je lahko NULL, v tem primeru se ne
    zabelezi, ce je bilo zamenjav liho stevilo, in se ne more racunati
    determinanta matrike.
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan deljitelj enak 0 (obicajno je ta kar 0!).
    Matriki res in d sta lahko isti kot m1, v tem primeru funkcija zapise
    v m1 diegonalne elemente d in poddiagonalne elemente matrike res, nad
    diagonalo pa zapise nicle.
      parity je lahko enak NULL, v tem primeru se ne zabelezi, ali je stevilo
    zamenjav bilo liho ali sodo (to se tako ali tako rabi samo pri racunanju
    determinante). Tudi indtab je naceloma lahko NULL, vendat to NI
    PRIPOROCLJIVO! V tem primeru namrec funkcija ne izvaja pivotiranja in se
    lahko zgodi, da ima matriko za singularno, ceprav ni. Drugace funkcija
    izvaja implicitno pivotiranje.
    Ref. Num. Rec., p.p.43-49.
    $A Igor avg00; */
{
int ret=0;
int i,j,k,n,imax,dim1,dim2;
double x,s,val;
double *line_m1_i,*line_m1_j,*line_l_i,*line_l_j,*line_u_i,*line_u_j,*line_v;
vector v=NULL;
dim1=m1->d1;
dim2=m1->d2;
n=m1->d1;
if (parity!=NULL)
  *parity=1;
if (ind!=NULL)
{
  /* Mozne so zamenjave vrstic, zato najprej poskrbimo, da se ohrani originalna
  matrika m1, ce je tako misljeno (torej, ce nobena od matrik ml in mu ni ista
  kot m1); ohranitev originalnih vrednosti dosezemo s tem, da m1 skopiramo v
  ml in uporabljamo to kopijo kot m1: */
  if (m1!=ml && m1!=mu)
  {
    for (i=1;i<=m1->d1;++i)
    {
      line_m1_i=m1->m[i];
      line_l_i=ml->m[i];
      for (j=1;j<=m1->d2;++j)
        line_l_i[j]=line_m1_i[j];
    }
    m1=ml;
  }
  v=getvector(n);
  line_v=v->v;
  /* Izracun skalirnih faktorjev za izbiro najboljsega pivota: */
  for (i=1;i<=n;++i)
  {
    val=0;
    line_m1_i=m1->m[i];
    for (j=1;j<=n;++j)
      if ((x=fabs(line_m1_i[j]))>val)
        val=x;
    if (val==0) /* V i. vrstici ni nenicelnega elementa, matrika je singularna */
    {
      dispvector(&v);
      return -1;
    } else
      line_v[i]=1/val;
  }
}
for (j=1;j<=n;++j) /* iteracija po stolpcih */
{
  line_m1_j=m1->m[j];
  line_l_j=ml->m[j];
  line_u_j=mu->m[j];  
  /* Izracun elementov nad diagonalo: */
  /* Izracun elementov nad diagonalo: */
  for (i=1;i<j;++i)
  {
    line_l_i=ml->m[i];
    s=m1->m[i][j];
    for (k=1;k<i;++k)
      s-=line_l_i[k]*mu->m[k][j];
    mu->m[i][j]=s;
  }
  /* Izracun elementov na in pod diagonalo in pomnjenje vrstice z relativno 
  najvecjim pivotom: */
  val=0; /* najvecji primerjalni faktor za pivot */
  imax=j; /* vrstica z najboljsim pivotom */
  for (i=j;i<=n;++i)
  {
    s=m1->m[i][j];
    line_l_i=ml->m[i];
    for (k=1;k<j;++k)
      s-=line_l_i[k]*mu->m[k][j];
    /* Prireditev izracunane vrednosti ustreznemu elementu ustrezne matrike: */
    if (i==j)
    {
      line_l_i[j]=1;
      mu->m[i][j]=s;
    } else
      line_l_i[j]=s;
    if (v!=NULL)
    {
      if ((x=v->v[i]*fabs(s)) >= val)
      {
        /* Nasli smo boljsi pivot od prejsnjega najboljsega */
        val=x;
        imax=i;
      }
    }
  }  /* iteracija po vrsticah - elementi na in pod diagonalo */
  /* Zamenjava vrstic, ce je to potrebno: */
  if (v!=NULL)
  {
    ind->t[j]=imax; /* indeks v tabeli, ki belezi zamenjave vrstic */
    if (imax!=j) /* zamenjava vrstic je potrebna */
    {
      /* printf("INTERCHANGING rows %i and %i.\n",j,imax); */
      /* Zamenjava vrstic j in imax: */
      line_l_i=ml->m[imax];
      line_u_i=mu->m[imax];
      for (k=1;k<j;++k)
      {
        x=line_l_j[k];
        line_l_j[k]=line_l_i[k];
        line_l_i[k]=x;
      }
      x=line_u_j[j];
      line_u_j[j]=line_l_i[j];
      line_l_i[j]=x;
      line_m1_i=m1->m[imax];
      for (k=j+1;k<=n;++k)
      {
        x=line_m1_j[k];
        line_m1_j[k]=line_m1_i[k];
        line_m1_i[k]=x;
      }
      /* Zaradi zamenjave se spremeni parnost, zamenjati je potrebno tudi
      skalirni faktor vrstice imax: */
      if (parity!=NULL)
        *parity=-*parity;
      line_v[imax]=line_v[j];  /* j. elementa ne rabimo vec */
    }
  }
  
  if (fabs(line_u_j[j])<=smalltol)
  {
    /* Zabelezen je nicti pivot, matrika je singularna */
    
  }
  /* Nazadnje se deljenje s pivotom: */
  if (j<n)
  {
    x=1/line_u_j[j];
    for (i=j+1;i<=n;++i)
      ml->m[i][j]*=x;
  }
} /* iteracija po stolpcih */
/* Ce se zgornje in spodnje trikotna matrika ne shranita v isto matriko,
postavimo naddiagonalne elemente pri ml in poddiagonalne pri mu na 0: */

if (ml!=mu)
  for (i=1;i<=n;++i)
  {
    line_l_i=ml->m[i];
    for (j=i+1;j<=n;++j)
      line_l_i[j]=mu->m[j][i]=0;
  }
dispvector(&v);
return ret;
}





matrix LUdecomptol0(matrix m1,matrix *resL,matrix *resU,indtab *ind,
                    int *parity,double smalltol)
    /* Vrne LU dekompozicijo matrike m1, t.j. produkt spodnjetrikotne matrike
    z enkami na diagonali in zgornjetrikotne matrike. Spodnjetrikotno matriko
    zapise v matriko *resL , ki jo tudi vrne, zgornjetrikotno pa v resU. resL
    ali resU sta lahko tudi naslova m1 ali naslova iste matrike. V tem primeru
    se enice na diagonali spodnjetrikotne matrike prepisejo z diagonalnimi
    elementi zgornjetrikotne matrike. V *indtab funkcija zabelezi vrstni red
    vrstic po zamenjavah, v *parity pa zapise -1, ce je bilo zamenjav liho st.,
    in 1, ce jih je bilo sodo stevilo (to se rabi pri racunanju determinante).
    indtab je lahko NULL, v tem primeru se dekompozicija izvede brez
    pivotiranja, vendar to NI PRIPOROCLJIVO, ker lahko dobimo 0 na diagonali,
    ceprav matrika ni singularna. parity je lahko NULL, v tem primeru se ne
    zabelezi, ce je bilo zamenjav liho stevilo, in se ne more racunati
    determinanta matrike.
    Funkcija vrne NULL, ce dekompozicija  ni bila uspesna (singularna matrika
    ali kak drug razlog).
    Funkcija uporablja konstanto smalltol za toleranco, s katero preveri, ce je
    izracunan deljitelj enak 0 (obicajno je kar 0!).
     OPOMBA:
     resL in resU sta lahko tudi NULL, v tem primeru funkcija vrne matriko, v
    kateri ima spravljeni obe trikotni matriki.
    $A Igor avg00; */
{
matrix m=NULL;
indtab indint;
if (resU==NULL)
  resU=resL;
else if (resL==NULL)
  resL=resU;
if (resL!=NULL)
{
  /* Ce je res!=NULL, se rezultat shrane v *res: */
  /* Najprej preverimo, ce je matrika m1 kompatibilna z operacijo: */
  if (m1==NULL)
  {
    dispmatrix(resL);
    dispmatrix(resU);
  }
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
  {
    dispmatrix(resL);
    dispmatrix(resU);
  }
  else
  {
    /* Preverimo dimenzijo *resL in *resU, ce ni ustrezna, ju zbrisemo: */
    if (*resL!=NULL)
      if ((*resL)->d1!=m1->d1 || (*resL)->d2!=m1->d2)
        dispmatrix(resL);
    /* Ce je *resL==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*resL==NULL)
      *resL=getmatrix(m1->d1,m1->d2);
    if (*resU!=NULL)
      if ((*resU)->d1!=m1->d1 || (*resU)->d2!=m1->d2)
        dispmatrix(resU);
    /* Ce je *resU==NULL, jo tvorimo z ustreznima dimenzijama: */
    if (*resU==NULL)
      *resU=getmatrix(m1->d1,m1->d2);
    if (ind!=NULL)
    {
      if (*ind!=NULL) /* Preverijo se dimenzije *ind */
        if ((*ind)->r<m1->d1 || (*ind)->r-(*ind)->ex>m1->d1)
          dispindtab(ind);
      if (*ind==NULL) /* Po potrebi se *ind alocira na novo */
        *ind=newindtab(0,m1->d1);
      (*ind)->n=m1->d1;
      indint=*ind; /* V operaciji bomo uporabljali indint, ki je NULL, ce je ind=NULL. */
    }
    /* Izvedba operacije: */
    if (LUdecomptolplain(m1,*resL,*resU,indint,parity,smalltol))
    {
      /* Operacija je bila neuspesna: */
      dispmatrix(resL); 
      dispmatrix(resU);
      dispindtab(ind);
      return NULL;
    }
  }
  return *resL;
} else
{
  /* resU=resL=NULL, naredi se nova matrika, v katero se zapise rezultat
  operacije, vrne se njen kazalec: */
  /* Najprej preverimo, ce je m1 kompatibilna z operacijo: */
  if (m1==NULL)
    return NULL;
  else if (m1->d1<1 || m1->d2<1 || m1->d1!=m1->d2)
    return NULL;
  else
  {
    m=getmatrix(m1->d1,m1->d2);
    /* Izvedba operacije: */
    if (ind!=NULL)
    {
      if (*ind!=NULL) /* Preverijo se dimenzije *ind */
        if ((*ind)->r<m1->d1 || (*ind)->r-(*ind)->ex>m1->d1)
          dispindtab(ind);
      if (*ind==NULL) /* Po potrebi se *ind alocira na novo */
        *ind=newindtab(0,m1->d1);
      (*ind)->n=m1->d1;
      indint=*ind; /* V operaciji bomo uporabljali indint, ki je NULL, ce je ind=NULL. */
    }
    if (LUdecomptolplain(m1,m,m,indint,parity,smalltol))
    {
      dispmatrix(&m);
      dispindtab(ind);
    }
    return m;
  }
}
}






            /*****************************************/
            /*                                       */
            /*   GRAMM-SCHMIDT's ORTHOGONALIZATION   */
            /*                                       */
            /*****************************************/


void GSortplain(matrix v,matrix q,matrix a)
    /* Gramm-Schmidtova ortogonalizacija. Stolpci matrike v so originalni
    vektorji, v matriko q se zapisejo ortogonalni vektorji dobljeni iz teh,
    v matriko a pa (desni) koeficienti, tako da je q=v*a. q postane ortogonalna
    matrika, a pa zgornjetrikotna z enkami na glavni diagonali.
      Dimenzije so naslednje: v n*m, q n*m, a m*m, kjer je n dimenzija
    vektorskega prostora, m pa stevilo vektorjev. Stolpci matrike a so
    vektorji q izrazeni kot linearne kombinacije vektorjev v.
      Matrika a je lahko NULL, v tem primeru se ne izracunajo koeficienti
    linearnih kombinacij.
      POZOR:
    Matriki q in a ne smeta biti isti kot v.
    $A Igor jul00; */
{
int i,j,k,l,m,n;
double x;
vector qnorms=NULL;
n=v->d1;  /* dimenzija prostora */
m=v->d2;  /* stevilo vektorjev */
identmat0(m,&a);
qnorms=getvector(m);
qnorms->v[1]=0;
/* 1. ortogon. vektor je enak 1. vektorju: */
for (i=1;i<=n;++i)
{
  q->m[i][1]=(x=v->m[i][1]);
  qnorms->v[1]+=x*x;
}
for (j=2;j<=m;++j)  /* tu j vedno tece po vektorjih, i pa po komponentah */
{
  for (i=1;i<=n;++i)
    q->m[i][j]=v->m[i][j];
  for (l=1;l<j;++l)
  {
    x=0;
    for (i=1;i<=n;++i)
      x+=v->m[i][j]*q->m[i][l]; /* Skal. prod. j. vekt. v in l. q */
    x/=qnorms->v[l];
    for (i=1;i<=n;++i)
      q->m[i][j]-=x*q->m[i][l];
    if (a!=NULL)  /* popravek koeficientov a */
      for (k=1;k<=l;++k)
      {
        a->m[k][j]-=x*a->m[k][l];
      }
  }
  if (j<m)
  {
    qnorms->v[j]=0;
    for (i=1;i<=n;++i)  /* Izracun kvadrata norme j. normalnega vektorja */
    {
      x=q->m[i][j];
      qnorms->v[j]+=x*x;
    }
  }
}
dispvector(&qnorms);
}


void GSortnormplain(matrix v,matrix q,matrix a)
    /* Gramm-Schmidtova ortogonalizacija z normiranjem. Stolpci matrike v so
    originalni vektorji, v matriko q se zapisejo ortonormirani vektorji
    dobljeni iz teh, v matriko a pa (desni) koeficienti, tako da je q=v*a.
    q postane ortonormalna atrika, a pa zgornjetrikotna z enkami na glavni
    diagonali.
      Dimenzije so naslednje: v n*m, q n*m, a m*m, kjer je n dimenzija
    vektorskega prostora, m pa stevilo vektorjev. Stolpci matrike a postanejo
    vektorji q izrazeni kot linearne kombinacije vektorjev v.
      Matrika a je lahko NULL, v tem primeru se ne izracunajo koeficienti
    linearnih kombinacij.
    $A Igor jul00; */
{
int i,j,k,l,m,n;
double x;
double norm;
n=v->d1;  /* dimenzija prostora */
m=v->d2;  /* stevilo vektorjev */
identmat0(m,&a);
norm=0;
/* 1. ortogon. vektor je enak 1. vektorju: */
for (i=1;i<=n;++i)
{
  q->m[i][1]=(x=v->m[i][1]);
  norm+=x*x;
}
norm=sqrt(norm);
for (i=1;i<=n;++i)
{
  q->m[i][1]/=norm;
}
if (a!=NULL)
  a->m[1][1]/=norm;
for (j=2;j<=m;++j)  /* tu j vedno tece po vektorjih, i pa po komponentah */
{
  for (i=1;i<=n;++i)
    q->m[i][j]=v->m[i][j];
  for (l=1;l<j;++l)
  {
    x=0;
    for (i=1;i<=n;++i)
      x+=v->m[i][j]*q->m[i][l]; /* Skal. prod. j. vekt. v in l. q */
    for (i=1;i<=n;++i)
      q->m[i][j]-=x*q->m[i][l];
    if (a!=NULL)  /* popravek koeficientov a */
      for (k=1;k<=l;++k)
      {
        a->m[k][j]-=x*a->m[k][l];
      }
  }
  norm=0;
  for (i=1;i<=n;++i)  /* izracun kvadrata norme j. normalnega vektorja */
  {
    x=q->m[i][j];
    norm+=x*x;
  }
  norm=sqrt(norm);
  for (i=1;i<=n;++i)  /* normiranje j. normalnega vektorja */
    q->m[i][j]/=norm;
  if (a!=NULL)
    for (k=1;k<=j;++k)
      a->m[k][j]/=norm;
}
}



void GSort0(matrix v,matrix *q,matrix *a)
    /* Gramm-Schmidtova ortogonalizacija. V matriko *q zapise po stolpcih
    ortogonalne vektorje dobljene iz stolpcev matrike v, v matriko *a pa
    ortogonalne vektorje kot linearne kombinacije stolpcev v (tudi po
    stolpcih). Ce je matrika v dimenzije n*m (m<=n), je *q dimenzije n*m in
    *a dimenzije m*m (m je stevilo vektorjev (stolpcev) v v, n pa je dimenzija
    vektorskega prostora).
     Ali q ali a je lahko NULL (ce je a NULL, se ustrezna matrika en izracuna,
    medtem ko se ort. vektorji v *q vedno izracunajo). a ali q je lahko tudi
    naslov v, v tem primeru se alocira nov prostor , originalni v pa zbrise po
    operaciji.
    $A Igor jul00; */
{
matrix Q=NULL,A=NULL,*saveq=NULL,*savea=NULL;
if (v==NULL)
{
  dispmatrix(q); dispmatrix(a);
} else if ((q==NULL && a==NULL) || v->d2<1 || v->d1<v->d2)
{
  dispmatrix(q); dispmatrix(a);
} else
{
  if (a==q)  /* a in q ne moreta zasedati istega prostora */
    a=NULL;
  if (q!=NULL)
  {
    /* Rezultat spravimo v *q: */
    /* Koda, ki preprecuje, da bi prislo do napacnih rezultatov, ce q kaze na
    v: */
    if (*q==v)
    {
      saveq=q;
      q=&Q;
    }
    /* Preverimo dimenzijo *q; ce ni ustrezna, zbrisemo matriko: */
    if (*q!=NULL)
      if ((*q)->d1!=v->d1 || (*q)->d2!=v->d2)
        dispmatrix(q);
    if (*q==NULL)
      *q=getmatrix(v->d1,v->d2);
    if (a==NULL)
      GSortplain(v,*q,NULL);
    else
    {
      /* V rezultatih se zahteva tudi matrika a: */
      /* Koda, ki preprecuje, da bi prislo do napacnih rezultatov, ce a kaze na
      v: */
      if (*a==v)
      {
        savea=a;
        a=&A;
      }
      /* Preverimo dimenzijo *a; ce ni ustrezna, zbrisemo matriko: */
      if (*a!=NULL)
        if ((*a)->d1!=v->d2 || (*a)->d2!=v->d2)
          dispmatrix(a);
      if (*a==NULL)
        *a=getmatrix(v->d2,v->d2);
      GSortplain(v,*q,*a);   /* Izvedba operacije */
      if (savea!=NULL) /* ce je *a==v */
      {
        dispmatrix(savea);
        *savea=*a;
      }
    }  /* a!=NULL) */
    if (saveq!=NULL) /* ce je *q==v */
    {
      dispmatrix(saveq);
      *saveq=*q;
    }
  } else  /* od q!=NULL */
  {
    /* q=NULL, funkcija naj vrne le koeficiente lin. komb. v *a. Vseeno moramo
    zacasno tvoriti matriko q, ker jo rabimo za izvedbo operacije: */
    Q=getmatrix(v->d1,v->d2);
    /* a!=NULL zaradi prej preverjenih pogojev. Spodnja koda preprecuje, da bi
    prislo do napacnih rezultatov, ce a kaze na v: */
    if (*a==v)
    {
      savea=a;
      a=&A;
    }
    /* Preverimo dimenzijo *a; ce ni ustrezna, zbrisemo matriko: */
    if (*a!=NULL)
      if ((*a)->d1!=v->d2 || (*a)->d2!=v->d2)
        dispmatrix(a);
    if (*a==NULL)
      *a=getmatrix(v->d2,v->d2);
    GSortplain(v,Q,*a);   /* Izvedba operacije */
    if (savea!=NULL) /* ce je *a==v */
    {
      dispmatrix(savea);
      *savea=*a;
    }
    dispmatrix(&Q);
  }
}
}



void GSortnorm0(matrix v,matrix *q,matrix *a)
    /* Gramm-Schmidtova ortogonalizacija z normalizacijo. V matriko *q zapise
    po stolpcih ortogonalne in normirane  vektorje dobljene iz stolpcev matrike
    v, v matriko *a pa ortogonalne vektorje kot linearne kombinacije stolpcev v
    (tudi po stolpcih). Ce je matrika v dimenzije n*m (m<=n), je *q dimenzije
    n*m in *a dimenzije m*m (m je stevilo vektorjev (stolpcev) v v, n pa je
    dimenzija vektorskega prostora).
     Ali q ali a je lahko NULL (ce je a NULL, se ustrezna matrika en izracuna,
    medtem ko se ort. vektorji v *q vedno izracunajo). a ali q je lahko tudi
    naslov v, v tem primeru se alocira nov prostor , originalni v pa zbrise po
    operaciji.
    $A Igor jul00; */
{
matrix Q=NULL,A=NULL,*saveq=NULL,*savea=NULL;
if (v==NULL)
{
  dispmatrix(q); dispmatrix(a);
} else if ((q==NULL && a==NULL) || v->d2<1 || v->d1<v->d2)
{
  dispmatrix(q); dispmatrix(a);
} else
{
  if (a==q)  /* a in q ne moreta zasedati istega prostora */
    a=NULL;
  if (q!=NULL)
  {
    /* Rezultat spravimo v *q: */
    /* Koda, ki preprecuje, da bi prislo do napacnih rezultatov, ce q kaze na
    v: */
    if (*q==v)
    {
      saveq=q;
      q=&Q;
    }
    /* Preverimo dimenzijo *q; ce ni ustrezna, zbrisemo matriko: */
    if (*q!=NULL)
      if ((*q)->d1!=v->d1 || (*q)->d2!=v->d2)
        dispmatrix(q);
    if (*q==NULL)
      *q=getmatrix(v->d1,v->d2);
    if (a==NULL)
      GSortnormplain(v,*q,NULL);
    else
    {
      /* V rezultatih se zahteva tudi matrika a: */
      /* Koda, ki preprecuje, da bi prislo do napacnih rezultatov, ce a kaze na
      v: */
      if (*a==v)
      {
        savea=a;
        a=&A;
      }
      /* Preverimo dimenzijo *a; ce ni ustrezna, zbrisemo matriko: */
      if (*a!=NULL)
        if ((*a)->d1!=v->d2 || (*a)->d2!=v->d2)
          dispmatrix(a);
      if (*a==NULL)
        *a=getmatrix(v->d2,v->d2);
      GSortnormplain(v,*q,*a);   /* Izvedba operacije */
      if (savea!=NULL) /* ce je *a==v */
      {
        dispmatrix(savea);
        *savea=*a;
      }
    }  /* a!=NULL) */
    if (saveq!=NULL) /* ce je *q==v */
    {
      dispmatrix(saveq);
      *saveq=*q;
    }
  } else  /* od q!=NULL */
  {
    /* q=NULL, funkcija naj vrne le koeficiente lin. komb. v *a. Vseeno moramo
    zacasno tvoriti matriko q, ker jo rabimo za izvedbo operacije: */
    Q=getmatrix(v->d1,v->d2);
    /* a!=NULL zaradi prej preverjenih pogojev. Spodnja koda preprecuje, da bi
    prislo do napacnih rezultatov, ce a kaze na v: */
    if (*a==v)
    {
      savea=a;
      a=&A;
    }
    /* Preverimo dimenzijo *a; ce ni ustrezna, zbrisemo matriko: */
    if (*a!=NULL)
      if ((*a)->d1!=v->d2 || (*a)->d2!=v->d2)
        dispmatrix(a);
    if (*a==NULL)
      *a=getmatrix(v->d2,v->d2);
    GSortnormplain(v,Q,*a);   /* Izvedba operacije */
    if (savea!=NULL) /* ce je *a==v */
    {
      dispmatrix(savea);
      *savea=*a;
    }
    dispmatrix(&Q);
  }
}
}






            /****************************/
            /*                          */
            /*   SYSTEMS OF EQUATIONS   */
            /*                          */
            /****************************/






    /* SOLUTION OF SYSTEMS WITH THE LDLT DECOMPOSITION: */


void solveLDLTplain(matrix L,matrix D,vector b,vector x)
    /* Solves the system of equations L D LT x = b, where L is a lower
    triangular matrix with diagonal elemnts 1, D is a diagonal matrix, and
    stores the result in x. Arguments must be allocated and of consistent
    dimensions since this is not checked.
      L and D can point to the same matrix since the function does not operate
    on diagonal elements of L (it just assumes these are 1).
      b and x can also point to the same vector.
    $A Igor nov03; */
{
int i,k,dim;
/* Remark: The order of solution is L y = b, D z = y, LT x = z */
dim=x->d;
/* Solve L y = b: */
x->v[1]=b->v[1];
for (i=2;i<=dim;++i)
{
  x->v[i]=b->v[i];
  for (k=1;k<i;++k)
    x->v[i]-=L->m[i][k]*x->v[k];
}
/* Solve D z = =y */
for (i=1;i<=dim;++i)
  x->v[i]/=D->m[i][i];
/* Solve LT x = z: */
for (i=dim-1;i>0;--i)
  for (k=i+1;k<=dim;++k)
    x->v[i]-=L->m[k][i]*x->v[k];   /* coef. of LT obtained from L (sym.) */
}


void solveLLTplain(matrix L,vector b,vector x)
    /* Solves the system of equations L LT x = b, where L is a lower
    triangular matrix, and stores the result in x. Arguments must be allocated
    and of consistent dimensions since this is not checked.
      b and x can point to the same vector.
      The function is usually used for back-substitution after the choleski
    (LLT) decomposition for positive definite matrices. A system of equations
    with a symmetric matrix A can be converted to a system with a positive
    definite matrix by multiplying the matrix ant the right-hand side vector
    by AT: A x = b => (AT A) x = (AT b). This is however not feasible because
    preparation of the system takes much more time than the solution, and it
    is much better to perform the LDLT deocmposition.
    $A Igor nov03; */
{
int i,k,dim;
/* Remark: The order of solution is L y = b, LT x = y */
dim=x->d;
/* Solve L y = b: */
x->v[1]=b->v[1]/L->m[1][1];
for (i=2;i<=dim;++i)
{
  x->v[i]=b->v[i];
  for (k=1;k<i;++k)
    x->v[i]-=L->m[i][k]*x->v[k];
  x->v[i]/=L->m[i][i];
}
/* Solve LT x = y: */
x->v[dim]/=L->m[dim][dim];
for (i=dim-1;i>0;--i)
{
  for (k=i+1;k<=dim;++k)
    x->v[i]-=L->m[k][i]*x->v[k];   /* coef. of LT obtained from L (sym.) */
  x->v[i]/=L->m[i][i];
}
}


    /* SOLUTION OF SYSTEMS WITH LU DECOMPOSITION: */


solvLUplain(matrix ml,matrix mu,indtab it,vector b,vector x)
    /* Resi sistem (ml*mu)x=b, kjer vsebujeta matriki ml in mu LU dekompozicijo
    matrike sistema, in resitev zapise v vektor x, ki je lahko tudi
    isti vektor kot vektor desnih strani b. L mora biti spodnjetrikotna z
    enicami na diagonali, U pa zgornjetrikotna matrika, dobljena z LU
    dekompozicijo, it pa je permutacijska tabela, v kateri so spravljene
    zamenjave vrstic. ml in mu sta lahko shranjeni tudi v isti matriki, ker se
    avtomaticno privzameta trikotnost obeh matrik in enice na diagonali pri ml.
     it je lahko tudi NULL, v tem primeru se privzame, da pri dekompoziciji ni
    bilo zamenjav vrstic. to se NIKOLI NE UPORABLJA, ker so v splosnem
    zamenjave vrstic nujne.
    $A Igor avg00; */
{
int i,j,firsnonzero=0,n;
double s;
n=b->d;
/* Ce se mora vektor b ohraniti (t.j. ce b ni isti vektor kot x), se najprej
b prepise v x: */
if (b!=x)
  for (i=1;i<=n;++i)
    x->v[i]=b->v[i];
/* 1. stopnja je resevanje sistema Ly=b (forward substitution), y se shrane v
x, kjer je ze kopija b: */
if (it==NULL) /* ce ni bilo zamenjav vrstic */
  for (i=1;i<=n;++i) /* treba je razvozlati zamenjave vrstic */
  {
    /* Zamenjava vrstic v vektorju b */
    s=x->v[i];
    if (firsnonzero)
      for (j=firsnonzero;j<=i-1;++j)
        s-=ml->m[i][j]*x->v[j];
    else if (s)
      firsnonzero=i;
    x->v[i]=s; /* Ker so diagonalni elementi od ml enaki 1, ni deljenja */
  }
else
  for (i=1;i<=n;++i) /* treba je razvozlati zamenjave vrstic */
  {
    /* Zamenjava vrstic v vektorju b */
    s=x->v[it->t[i]];
    x->v[it->t[i]]=x->v[i];
    if (firsnonzero)
      for (j=firsnonzero;j<=i-1;++j)
        s-=ml->m[i][j]*x->v[j];
    else if (s)
      firsnonzero=i;
    x->v[i]=s; /* Ker so diagonalni elementi od ml enaki 1, ni deljenja */
  }
/* Resevanje sistema Ux=y (backsubstitution), y je shranjen v x: */
for (i=n;i>=1;--i)
{
  s=x->v[i];
  for (j=i+1;j<=n;++j)
    s-=mu->m[i][j]*x->v[j];
  x->v[i]=s/mu->m[i][i];
}
}




vector solvLU0(matrix ml,matrix mu,indtab it,vector b2,vector *x3)
    /* Vrne resitev sistema enacb (ml*mu)*x3=b2, kjer sta ml in mu matriki
    dobljeni z LU dekompozicijo matrike sistema, b2 pa vektor desnih strani.
    it je indeksna tabela, v kateri so zabelezene izvedene zamenjave vrstic
    pri dekompoziciji. Lahko je NULL, ce smo dekompozicijo na siloizvedli
    tako, da ni bilo zamenjav, vendar se to v praksi NE SME narediti, ker
    lahko na ta nacin dekompozicija spodleti tudi pri nesingularnih matrikah.
    Ce je x3 razlicen od NULL, zapise rezultat v *x3 in vrne *x3. x3 je lahko
    tudi naslov b2, funkcija v tem primeru poskrbi za to, da se najprej
    izracuna rezultat operacije in sele nato prepise operand. ml in mu sta
    lahko shranjeni v isti matriki, ker se avtomatsko privzame, da ima ml enice
    na diagonali. Ce je ml ali mu NULL, se vzame, da sta ml in mu shranjeni v
    isti matriki in se za obe uporabi tista, ki je razlicna od NULL.
     OPOMBA:
     Ce je x3 razlicen od NULL, se po navadi vektor, ki ga funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze x3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor avg00; */
{
vector v=NULL;
/* Ce je ml ali mu enak NULL, se privzame, da sta oba faktorja shranjena v isti
matriki: */
if (ml==NULL)
  ml=mu;
else if (mu==NULL)
  mu=ml;
if (x3!=NULL)
{
  /* Ce je x3!=NULL, se rezultat shrane v *x3. Najprej preverimo, ce so ml, mu,
  it in b2 kompatibilni z operacijo: */
  if (ml==NULL || mu==NULL || b2==NULL)
    dispvector(x3);
  else if (ml->d1!=b2->d || ml->d2!=b2->d ||
   mu->d1!=b2->d || mu->d2!=b2->d || b2->d<1)
    dispvector(x3);
  else
  {
    /* Preverimo dimenzijo *x3, ce ni ustrezna, zbrisemo *x3: */
    if (*x3!=NULL)
      if ((*x3)->d!=b2->d)
        dispvector(x3);
    /* Ce je *x3==NULL, tvorimo *x3 z dimenzijama ml: */
    if (*x3==NULL)
      *x3=getvector(b2->d);
    /* Izvedba operacije, preveri se se, ce ima it pravilne dimenzije: */
    if (it==NULL)
      solvLUplain(ml,mu,it,b2,*x3);
    else if (it->n==b2->d)
      solvLUplain(ml,mu,it,b2,*x3);
    else
      dispvector(x3);
  }
  return *x3; /* Ker je bil x3!=NULL, se vrne *x3. */
} else   /* x3=NULL */
{
  /* x3==NULL, naredi se nova vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce sta ml in b2 kompatibilni za operacijo: */
  if (ml==NULL || mu==NULL || b2==NULL)
    return NULL;
  else if (ml->d1!=b2->d || ml->d2!=b2->d ||
   mu->d1!=b2->d || mu->d2!=b2->d || b2->d<1)
    return NULL;
  else
  {
    /* Izvedba operacije, preveri se se, ce ima it pravilne dimenzije: */
    if (it==NULL)
    {
      v=getvector(b2->d);
      solvLUplain(ml,mu,it,b2,v);
    } else if (it->n==b2->d)
    {
      v=getvector(b2->d);
      solvLUplain(ml,mu,it,b2,v);
    }
    return v;
  }
}
}


solvLUmatplain(matrix ml,matrix mu,indtab it,matrix b,matrix mx)
    /* Resi sistem (ml*mu)x=b, kjer vsebujeta matriki ml in mu LU dekompozicijo
    matrike sistema, in resitev zapise v vektor mx, ki je lahko tudi
    isti vektor kot vektor desnih strani b. L mora biti spodnjetrikotna z
    enicami na diagonali, U pa zgornjetrikotna matrika, dobljena z LU
    dekompozicijo, it pa je permutacijska tabela, v kateri so spravljene
    zamenjave vrstic. ml in mu sta lahko shranjeni tudi v isti matriki, ker se
    avtomaticno privzameta trikotnost obeh matrik in enice na diagonali pri ml.
     it je lahko tudi NULL, v tem primeru se privzame, da pri dekompoziciji ni
    bilo zamenjav vrstic. to se NIKOLI NE UPORABLJA, ker so v splosnem
    zamenjave vrstic nujne.
    $A Igor avg00; */
{
int i,j,k,firsnonzero,n,m;
double s;
n=b->d1;
m=b->d2;
/* Ce se mora matrika b ohraniti (t.j. ce b ni ista matrika kot mx), se najprej
b prepise v mx: */
if (b!=mx)
  for (i=1;i<=n;++i)
    for (k=1;k<=m;++k)
      mx->m[i][k]=b->m[i][k];
for (k=1;k<=m;++k) /* Iteracija po stolpcih desnih strani */
{
  firsnonzero=0;
  /* 1. stopnja je resevanje sistema Ly=b (forward substitution), y se shrane v
  mx, kjer je ze kopija b: */
  if (it==NULL) /* ce ni bilo zamenjav vrstic */
    for (i=1;i<=n;++i) /* treba je razvozlati zamenjave vrstic */
    {
      /* Zamenjava vrstic v vektorju b */
      s=mx->m[i][k];
      if (firsnonzero)
        for (j=firsnonzero;j<=i-1;++j)
          s-=ml->m[i][j]*mx->m[j][k];
      else if (s)
        firsnonzero=i;
      mx->m[i][k]=s; /* Ker so diagonalni elementi ml enaki 1, ni deljenja */
    }
  else
    for (i=1;i<=n;++i) /* treba je razvozlati zamenjave vrstic */
    {
      /* Zamenjava vrstic v vektorju b */
      s=mx->m[it->t[i]][k];
      mx->m[it->t[i]][k]=mx->m[i][k];
      if (firsnonzero)
        for (j=firsnonzero;j<=i-1;++j)
          s-=ml->m[i][j]*mx->m[j][k];
      else if (s)
        firsnonzero=i;
      mx->m[i][k]=s; /* Ker so diagonalni elementi od ml enaki 1, ni deljenja */
    }
  /* Resevanje sistema Ux=y (backsubstitution), y je shranjen v mx: */
  for (i=n;i>=1;--i)
  {
    s=mx->m[i][k];
    for (j=i+1;j<=n;++j)
      s-=mu->m[i][j]*mx->m[j][k];
    mx->m[i][k]=s/mu->m[i][i];
  }
}
}





    /* SOLUTION OF SYSTEMS OF EQUATIONS WITH ORTHOGONAL MATRICES: */



solvortplain(matrix q,vector b,vector x)
    /* Resi sistem enacb q*x=b, kjer je q ortogonalna matrika (stolpci matrike
    so ortogonalni), b pa vektor desnih strani. Resitev se zapise v vektor x.
    Funkcija ne kontrolira dimenzij.
    x ne sme biti isti vektor kot b.
    $A Igor jul00; */
{
int i,k,n;
double qnorm,scalprod;
n=q->d1;
for (i=1;i<=n;++i) /* iteracija po stolpcih q oz. vrsticah x */
{
  /* Norma i. stolpca q: */
  qnorm=0;
  for (k=1;k<=n;++k)
  {
    scalprod=q->m[k][i]; /* pomozna sprem. */
    qnorm+=scalprod*scalprod;
  }
    scalprod=0;
    for (k=1;k<=n;++k)
      scalprod+=q->m[k][i]*b->v[k];
    x->v[i]=scalprod/qnorm;
}
}


solvortmatplain(matrix q,matrix b,matrix x)
    /* Resi sistem enacb q*x=b, kjer je q ortogonalna matrika (stolpci matrike
    so ortogonalni), stolpci b pa so vektorji desnih strani. Resitev se zapise
    po stolpcih v matriko x. Funkcija ne kontrolira dimenzij.
    x ne sme biti ista matrika kot b.
    $A Igor jul00; */
{
int i,j,k,n,m;
double qnorm,scalprod;
n=q->d1;
m=x->d2;
for (i=1;i<=n;++i) /* iteracija po stolpcih q oz. vrsticah x */
{
  /* Norma i. stolpca q: */
  qnorm=0;
  for (j=1;j<=n;++j)
  {
    scalprod=q->m[j][i]; /* pomozna sprem. */
    qnorm+=scalprod*scalprod;
  }
  for (j=1;j<=m;++j) /* iteracija po stolpcih x in b */
  {
    scalprod=0;
    for (k=1;k<=n;++k)
      scalprod+=q->m[k][i]*b->m[k][j];
    x->m[i][j]=scalprod/qnorm;
  }
}
}



solvortnormplain(matrix q,vector b,vector x)
    /* Resi sistem enacb q*x=b, kjer je q ortonormalna matrika (stolpci matrike
    so paroma ortogonalni in normirani), b pa vektor desnih strani. Resitev se
    zapise v vektor x. Funkcija ne kontrolira dimenzij.
    x ne sme biti isti vektor kot b.
    $A Igor jul00; */
{
int i,k,n;
double scalprod;
n=q->d1;
for (i=1;i<=n;++i) /* iteracija po stolpcih q oz. vrsticah x */
{
  scalprod=0;
  for (k=1;k<=n;++k)
    scalprod+=q->m[k][i]*b->v[k];
  x->v[i]=scalprod;
}
}



solvortnormmatplain(matrix q,matrix b,matrix x)
    /* Resi sistem enacb q*x=b, kjer je q ortonormalna matrika (stolpci matrike
    so ortogonalni in normirani), stolpci b pa so vektorji desnih strani.
    Resitev se zapise po stolpcih v matriko x. Funkcija ne kontrolira dimenzij.
    x ne sme biti ista matrika kot b.
    $A Igor jul00; */
{
int i,j,k,n,m;
double scalprod;
n=q->d1;
m=x->d2;
for (i=1;i<=n;++i) /* iteracija po stolpcih q oz. vrsticah x */
  for (j=1;j<=m;++j) /* iteracija po stolpcih x in b */
  {
    scalprod=0;
    for (k=1;k<=n;++k)
      scalprod+=q->m[k][i]*b->m[k][j];
    x->m[i][j]=scalprod;
  }
}




vector solvort0(matrix q1,vector b2,vector *x3)
    /* Vrne resitev sistema enacb q1*x3=b2, kjer je q1 ortogonalna matrika,
    t.j. matrika s paroma ortogonalnimi stolpci. Ce je x3 razlicen od NULL,
    zapise rezultat v *x3 in vrne *x3. x3 je lahko tudi naslov b2, funkcija
    v tem primeru poskrbi za to, da se najprej izracuna rezultat operacije
    in sele nato prepise operand.
     OPOMBA:
     Ce je x3 razlicen od NULL, se po navadi vektor, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze x3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor avg00; */
{
vector v=NULL,*save=NULL;
if (x3!=NULL)
{
  if (*x3==b2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce x3 kaze na
    b2: */
    save=x3;
    x3=&v;
  }
  /* Ce je x3!=NULL, se rezultat shrane v *x3. Najprej preverimo, ce sta q1
  in b2 kompatibilni z operacijo: */
  if (q1==NULL || b2==NULL)
    dispvector(x3);
  else if (q1->d1!=b2->d || q1->d2!=b2->d || b2->d<1)
    dispvector(x3);
  else
  {
    /* Preverimo dimenzijo *x3, ce ni ustrezna, zbrisemo *x3: */
    if (*x3!=NULL)
      if ((*x3)->d!=b2->d)
        dispvector(x3);
    /* Ce je *x3==NULL, tvorimo *x3 z dimenzijama q1: */
    if (*x3==NULL)
      *x3=getvector(b2->d);
    /* Izvedba operacije: */
    solvortplain(q1,b2,*x3);
  }
  if (save!=NULL)
  {
    dispvector(save);
    *save=*x3;
  }
  return *x3; /* Ker je bil x3!=NULL, se vrne *x3. */
} else   /* x3=NULL */
{
  /* x3==NULL, naredi se nova vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce sta q1 in b2 kompatibilni za operacijo: */
  if (q1==NULL || b2==NULL)
    return NULL;
  else if (q1->d1!=b2->d || q1->d2!=b2->d || b2->d<1)
    return NULL;
  else
  {
    v=getvector(b2->d);
    /* Izvedba operacije: */
    solvortplain(q1,b2,v);
    return v;
  }
}
}



matrix solvortmat0(matrix q1,matrix b2,matrix *x3)
    /* Vrne resitve sistemov enacb q1*x3=b2, kjer je q1 ortogonalna matrika,
    t.j. matrika s paroma ortogonalnimi stolpci. Ce je x3 razlicen od NULL,
    zapise rezultat v *x3 in vrne *x3. x3 je lahko tudi naslov q1 ali b2,
    funkcija v tem primeru poskrbi za to, da se najprej izracuna rezultat
    operacije in sele nato prepise operand.
     OPOMBA:
     Ce je x3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze x3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor avg00; */
{
matrix m=NULL,*save=NULL;
if (x3!=NULL)
{
  if (*x3==q1 || *x3==b2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce x3 kaze na
    q1 ali b2: */
    save=x3;
    x3=&m;
  }
  /* Ce je x3!=NULL, se rezultat shrane v *x3. Najprej preverimo, ce sta q1
  in b2 kompatibilni z operacijo: */
  if (q1==NULL || b2==NULL)
    dispmatrix(x3);
  else if (q1->d1!=b2->d1 || q1->d2!=b2->d1 || b2->d1<1 || b2->d2<1)
    dispmatrix(x3);
  else
  {
    /* Preverimo dimenzijo *x3, ce ni ustrezna, zbrisemo *x3: */
    if (*x3!=NULL)
      if ((*x3)->d1!=b2->d1 || (*x3)->d2!=b2->d2)
        dispmatrix(x3);
    /* Ce je *x3==NULL, tvorimo *x3 z dimenzijama q1: */
    if (*x3==NULL)
      *x3=getmatrix(b2->d1,b2->d2);
    /* Izvedba operacije: */
    solvortmatplain(q1,b2,*x3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*x3;
  }
  return *x3; /* Ker je bil x3!=NULL, se vrne *x3. */
} else   /* x3=NULL */
{
  /* x3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta q1 in b2 kompatibilni za operacijo: */
  if (q1==NULL || b2==NULL)
    return NULL;
  else if (q1->d1!=b2->d1 || q1->d2!=b2->d1 || b2->d1<1 || b2->d2<1)
    return NULL;
  else
  {
    m=getmatrix(b2->d1,b2->d2);
    /* Izvedba operacije: */
    solvortmatplain(q1,b2,m);
    return m;
  }
}
}



vector solvortnorm0(matrix q1,vector b2,vector *x3)
    /* Vrne resitev sistema enacb q1*x3=b2, kjer je q1 ortogonalna matrika,
    t.j. matrika s paroma ortogonalnimi stolpci. Ce je x3 razlicen od NULL,
    zapise rezultat v *x3 in vrne *x3. x3 je lahko tudi naslov b2, funkcija
    v tem primeru poskrbi za to, da se najprej izracuna rezultat operacije
    in sele nato prepise operand.
     OPOMBA:
     Ce je x3 razlicen od NULL, se po navadi vektor, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi vektorju, na katerega kaze x3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isti vektor).
    $A Igor avg00; */
{
vector v=NULL,*save=NULL;
if (x3!=NULL)
{
  if (*x3==b2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce x3 kaze na
    b2: */
    save=x3;
    x3=&v;
  }
  /* Ce je x3!=NULL, se rezultat shrane v *x3. Najprej preverimo, ce sta q1
  in b2 kompatibilni z operacijo: */
  if (q1==NULL || b2==NULL)
    dispvector(x3);
  else if (q1->d1!=b2->d || q1->d2!=b2->d || b2->d<1)
    dispvector(x3);
  else
  {
    /* Preverimo dimenzijo *x3, ce ni ustrezna, zbrisemo *x3: */
    if (*x3!=NULL)
      if ((*x3)->d!=b2->d)
        dispvector(x3);
    /* Ce je *x3==NULL, tvorimo *x3 z dimenzijama q1: */
    if (*x3==NULL)
      *x3=getvector(b2->d);
    /* Izvedba operacije: */
    solvortnormplain(q1,b2,*x3);
  }
  if (save!=NULL)
  {
    dispvector(save);
    *save=*x3;
  }
  return *x3; /* Ker je bil x3!=NULL, se vrne *x3. */
} else   /* x3=NULL */
{
  /* x3==NULL, naredi se nova vektor, v katerega se zapise rezultat operacije,
  vrne se njegov kazalec: */
  /* Najprej preverimo, ce sta q1 in b2 kompatibilni za operacijo: */
  if (q1==NULL || b2==NULL)
    return NULL;
  else if (q1->d1!=b2->d || q1->d2!=b2->d || b2->d<1)
    return NULL;
  else
  {
    v=getvector(b2->d);
    /* Izvedba operacije: */
    solvortnormplain(q1,b2,v);
    return v;
  }
}
}


matrix solvortnormmat0(matrix q1,matrix b2,matrix *x3)
    /* Vrne resitve sistemov enacb q1*x3=b2, kjer je q1 ortogonormirana matrika,
    t.j. matrika s paroma ortogonalnimi in normiranimi stolpci (v evklidski
    normi). Ce je x3 razlicen od NULL, zapise rezultat v *x3 in vrne *x3. x3
    je lahko tudi naslov q1 ali b2, funkcija v tem primeru poskrbi za to, da
    se najprej izracuna rezultat operacije in sele nato prepise operand.
     OPOMBA:
     Ce je x3 razlicen od NULL, se po navadi matrika, ki jo funkcija vrne, ne
    priredi nicemur ali se priredi matriki, na katero kaze x3 (ker dobimo
    drugace situacijo, ko dva kazalca kazeta na isto matriko).
    $A Igor avg00; */
{
matrix m=NULL,*save=NULL;
if (x3!=NULL)
{
  if (*x3==q1 || *x3==b2)
  {
    /* Koda, ki prepreci, da bi prislo do napacnih rezultatov, ce x3 kaze na
    q1 ali b2: */
    save=x3;
    x3=&m;
  }
  /* Ce je x3!=NULL, se rezultat shrane v *x3. Najprej preverimo, ce sta q1
  in b2 kompatibilni z operacijo: */
  if (q1==NULL || b2==NULL)
    dispmatrix(x3);
  else if (q1->d1!=b2->d1 || q1->d2!=b2->d1 || b2->d1<1 || b2->d2<1)
    dispmatrix(x3);
  else
  {
    /* Preverimo dimenzijo *x3, ce ni ustrezna, zbrisemo *x3: */
    if (*x3!=NULL)
      if ((*x3)->d1!=b2->d1 || (*x3)->d2!=b2->d2)
        dispmatrix(x3);
    /* Ce je *x3==NULL, tvorimo *x3 z dimenzijama q1: */
    if (*x3==NULL)
      *x3=getmatrix(b2->d1,b2->d2);
    /* Izvedba operacije: */
    solvortnormmatplain(q1,b2,*x3);
  }
  if (save!=NULL)
  {
    dispmatrix(save);
    *save=*x3;
  }
  return *x3; /* Ker je bil x3!=NULL, se vrne *x3. */
} else   /* x3=NULL */
{
  /* x3==NULL, naredi se nova matrika, v katero se zapise rezultat operacije,
  vrne se njen kazalec: */
  /* Najprej preverimo, ce sta q1 in b2 kompatibilni za operacijo: */
  if (q1==NULL || b2==NULL)
    return NULL;
  else if (q1->d1!=b2->d1 || q1->d2!=b2->d1 || b2->d1<1 || b2->d2<1)
    return NULL;
  else
  {
    m=getmatrix(b2->d1,b2->d2);
    /* Izvedba operacije: */
    solvortnormmatplain(q1,b2,m);
    return m;
  }
}
}


vector solvGS0(matrix M,vector b,vector *x)
    /* Resi sistem enacb M*x=b tako, da najprej izvede normirano
    Gramm-Schmidtovo ortogonalizacijo stolpcev M in v obliki Q=MA, nato
    pa resi sistem v dveh stopnjah, najprej resi Q*y=b, nato pa izracuna x kot
    x=A*y.
     M mora biti kvadratna matrika in b vektor iste dimenzije kot M. x je lahko
    NULL, v tem primeru se na novo kreira in vrne vektor, ki vsebuje resitev.
    Ce je x!=NULL, se resitev zapise v *x (najprej se poskrbi, da ima *x isto
    dimenzijo kot b), *x pa se tudi vrne.
    $A Igor jul00; */
{
matrix Q=NULL,A=NULL;
vector y=NULL;
GSortnorm0(M,&Q,&A);
if (x!=NULL)
{
  solvortnorm0(Q,b,x);
  matprodvec0(A,*x,x);
  dispmatrix(&Q);
  dispmatrix(&A);
  return(*x);
} else  /* x=NULL */
{
  solvortnorm0(Q,b,&y);
  matprodvec0(A,y,&y);
  dispmatrix(&Q);
  dispmatrix(&A);
  return y;
}
}


matrix solvmatGS0(matrix M,matrix b,matrix *x)
    /* Resi sisteme enacb M*x=b tako, da najprej izvede normirano
    Gramm-Schmidtovo ortogonalizacijo stolpcev M in v obliki Q=MA, nato
    pa resi sistem v dveh stopnjah, najprej resi Q*y=b, nato pa izracuna x kot
    x=A*y.
     M mora biti kvadratna matrika in b matrika z istim stevilom vrstic, kot je
    dimenzija m, stolpcev pa mora imeti toliko, kolikor sistemov resujemo. x je
    lahko NULL, v tem primeru se na novo kreira in vrne vektor, ki vsebuje
    resitev. Ce je x!=NULL, se resitev zapise v *x (najprej se poskrbi, da ima
    *x iste dimenzije kot b), *x pa se tudi vrne.
    $A Igor jul00; */
{
matrix Q=NULL,A=NULL;
matrix y=NULL;
GSortnorm0(M,&Q,&A);
if (x!=NULL)
{
  solvortnormmat0(Q,b,x);
  matprod0(A,*x,x);
  dispmatrix(&Q);
  dispmatrix(&A);
  return(*x);
} else  /* x=NULL */
{
  solvortnormmat0(Q,b,&y);
  matprod0(A,y,&y);
  dispmatrix(&Q);
  dispmatrix(&A);
  return y;
}
}





            /*****************************/
            /*                           */
            /*   INVERSION OF MATRICES   */
            /*                           */
            /*****************************/



static void matinvortplain(matrix q,matrix inv)
    /* Izracuna inverz ortogonalne matrike q in rezultat zapise v inv. Funkcija
    ne preverja dimenzij. Matrika inv ne sme biti ista kot q.
    $A Igor jul00; */
{
int i,j,n;
double qnorm,x;
n=q->d1;
for (i=1;i<=n;++i)
{
  qnorm=0;
  for (j=1;j<=n;++j)  /* kvadrat norme i. vrstice */
  {
    x=q->m[j][i];
    qnorm+=x*x;
  }
  /* Izracun elementov i. stolpca inverzne matrike: */
  for (j=1;j<=n;++j)
    inv->m[i][j]=q->m[j][i]/qnorm;
}
}



matrix matinvGS0(matrix M,matrix *inv)
    /* Izracuna inverzno matriko matrike M s pomocjo Gramm-Schmidtove
    ortogonalizacije in jo vrne. Ce je inv!=NULL, vrne inverzno matriko v
    *inv, drugace inverzno matriko tvori na novo.
    $A Igor jul00; */
{
matrix Q=NULL,A=NULL;
matrix y=NULL;
GSortnorm0(M,&Q,&A);
mattransp0(Q,&Q);  /* Inverz Q je kar transponirana matrika Q */
if (inv!=NULL)
{
  matprod0(A,Q,inv);
  dispmatrix(&Q);
  dispmatrix(&A);
  return *inv;
} else  /* x=NULL */
{
  y=matprod0(A,Q,NULL);
  dispmatrix(&Q);
  dispmatrix(&A);
  return y;
}
}



